#include <afxwin.h>
#include "CWirelessSample.h"
#include "mactree.h"

#ifndef __CRFMAP__
#define __CRFMAP__

class CRFMap {
public:
  CRFMap(int sizex, int sizey);
  ~CRFMap();

  // File access functions

  // SetCurrentFile points the RF map to a different file,
  // returns true on success
  bool SetCurrentFile(char * fileName);

  // ReadFromFile returns true on a successful read from file,
  // false otherwise
  bool ReadFromFile();

  // AddSample returns true on success, false otherwise
  bool AddSample(int x, int y, CWirelessSample * sample);

  // Getposition puts the position into the passed CPoint
  // returns true on success, false on failure
  // mode = 0 -> Simple inc if mac found algorithm
  // mode = 1 -> Interpolate position from data available
  //             IE do some weighted averaging or something
  //             to find position based on "nearby" known
  //             points.
  // mode = 2 -> inc if mac found dec if not algorithm
  // mode = 3 -> choose based on APs heard, use signal strength
  //             to break ties
  // Write the position into the CPoint passed to the function
  bool GetPosition(CPoint * point, int mode, CWirelessSample * sample);

  // WriteToFile saves the CRFMap to the file
  bool WriteToFile();

  // OOPS! We screwed up, undo the last add to the RFMap
  // Not callable more than once,
  // IE UndoLastAdd(); UndoLastAdd(); first returns true,
  // second returns false, unless an AddSample() goes
  // between the two
  bool UndoLastAdd();

private:

  //mode 0 implementation of GetPosition
  //ALGORITHM: pick the first point encountered in the mac tree
  //           that matches the most mac addresses from the sample
  bool getPos0(CPoint *, CWirelessSample *);

  //mode 2 implementation of GetPosition
  //ALGORITHM: pick all the points encountered in the mac tree
  //           that match the most mac addresses from the sample
  //           then choose the first one in the list with the least
  //           amount of APs at that point
  bool getPos2(CPoint *, CWirelessSample *);

  //mode 3 implementation of GetPosition
  //ALGORITHM: pick all the points encountered in the mac tree
  //           that match the most mac addresses from the sample
  //           then choose all the points in the list with the least
  //           amount of APs available at them, then choose the first
  //           point in the least with the least difference in signal
  //           strength for all the AP's that can be heard in the
  //           sample and the points in the list
  bool getPos3(CPoint *, CWirelessSample *);

  //mode 1 implementation of GetPosition
  //ALGORITHM: dan sucks
  bool getPos1(CPoint *, CWirelessSample *);

  //returns a list of all the possible points that it matched
  //to the sample given with their corresponding weights in
  //positionLL and the most likely in the CPoint passed to it
  //ALGORITHM: pick the first point encountered in the mac tree
  //           that matches the most mac addresses from the sample
  positionLL *getPosCounts(CPoint *, CWirelessSample *);

  //returns a positionLL containing all the points that are weighted
  //equal to the maximum weighted point in the positionLL passed
  positionLL *getMaxes(positionLL *);

  //weights the points in positionLL such that a point with less
  //access points are weighted more
  bool leastAPs(positionLL *);

  //weights the points in positionLL such that the closer the signal
  //strengths of the position and the sample are the more a point 
  //is weighted
  bool weightSignal(positionLL *, CWirelessSample *);

  void addWeight(positionLL *);
  void interpolate(positionLL *, CPoint *);
  bool getPos4(CPoint *, CWirelessSample *);
  void weightHistory(positionLL *);

  // put whatever you want here
  bool fileSet; // don't need to keep this
  macTree *myTree;
  bool undo;
  int ***macCount; //count of mac's for each x,y
  int xsize;
  int ysize;
  char *mapFile;
  fstream file;
  int lastx;
  int lasty;
};

#endif


