#include "CWirelessSample.h"
#include <afxsock.h> // windows only...for htonl and ntohl

// CWIRLESSSAMPLE FUNCTIONS
CWirelessSample::CWirelessSample() {
  // blind init
  SetChild(NULL);
  Reset();
}

CWirelessSample::CWirelessSample(CWirelessAPNode * newKid) {
  SetChild(newKid);
  Reset();
}

CWirelessSample::CWirelessSample(MsgToServer * msg) {
  if(msg==NULL) {
    CWirelessSample();
	return;
  }

  int nAPNodes = msg->header.length / 6;
  CWirelessAPNode * apnode = NULL;
  char mac[13];
  mac[12] = '\0';
  short channel;
  short signal;
  short noise;

  for(int i=0;i<nAPNodes;i++) {
	for(int j=0;j<12;j++)
	  mac[j] = msg->apnodes[i].mac[j];
	channel = (short) ntohl(msg->apnodes[i].channel);
	noise = (short) ntohl(msg->apnodes[i].noise);
	signal = (short) ntohl(msg->apnodes[i].noise);
	apnode = new CWirelessAPNode("APNODE",&mac[0],signal,noise,channel,apnode);
  }

  SetChild(apnode);
  Reset();

}

void CWirelessSample::SetChild(CWirelessAPNode * newKid) {
  myKid = newKid;
}

void CWirelessSample::Reset() {
  current = myKid;
}

bool CWirelessSample::AtEnd() {
  return current==NULL;
}

void CWirelessSample::Next() {
  if(!AtEnd()) {
    current = current->getSibling();
  }
}

void CWirelessSample::FreeMem() {
  if(myKid!=NULL) {
    myKid->FreeMem();
	delete myKid;
  }
  myKid = NULL;
  Reset();
}

CWirelessAPNode * CWirelessSample::Current() {
  return current;
}

void CWirelessSample::GenerateMessage(MsgToServer * msg) {

  // generate a header, not including the length, zeroing the type
  msg->header.proto = htonl(0x90200902);
  msg->header.type = htonl(0);

  // loop on the kids, adding them as we go
  Reset();
  int kidnumber = 0;
  while(!AtEnd()) {
    strcpy(msg->apnodes[kidnumber].mac,Current()->getMACAddr());
	msg->apnodes[kidnumber].channel = htonl(Current()->getChannel());
	msg->apnodes[kidnumber].signal  = htonl(Current()->getSignal());
	msg->apnodes[kidnumber].noise   = htonl(Current()->getNoise());
	kidnumber++;
	Next();
  }

  msg->header.length = htonl(6 * (kidnumber));

  Reset();

  return;
}


// CWIRELESSAPNODE FUNCTIONS

CWirelessAPNode::CWirelessAPNode() {
  // blind initialization
  setSignal(0);
  setNoise(0);
  setChannel(0);
  setSibling(NULL);
}

CWirelessAPNode::CWirelessAPNode(CWirelessAPNode * sibling) {
  // half-blind init
  setSignal(0);
  setNoise(0);
  setChannel(0);
  setSibling(sibling);

}

CWirelessAPNode::CWirelessAPNode(char * newName,
								 char * newMAC,
								 short newSignal,
								 short newNoise,
								 short newChannel,
								 CWirelessAPNode * sibling) {
  setSignal(newSignal);
  setNoise(newNoise);
  setChannel(newChannel);
  setSibling(sibling);

  // copy names in, not using strcpy() to avoid include
  setName(newName);
  setMACAddr(newMAC);

  
}

void CWirelessAPNode::setChannel(short newChannel) {
  myChannel = newChannel;
}

void CWirelessAPNode::setNoise(short newNoise) {
  myNoise = newNoise;
}

void CWirelessAPNode::setSignal(short newSignal) {
  mySignal = newSignal;
}

void CWirelessAPNode::setSibling(CWirelessAPNode * newSibling) {
  mySibling = newSibling;
}

void CWirelessAPNode::setName(char * newName) {
  int i;
  for(i=0;newName[i]!=NULL&&i<31;i++) myName[i] = newName[i];
  myName[i] = NULL; // Null-terminate
}

void CWirelessAPNode::setMACAddr(char * newMac) {
  int i;
  for(i=0;newMac[i]!=NULL&&i<15;i++) myMac[i] = newMac[i];
  myMac[i] = NULL; // Null-terminate
}

char * CWirelessAPNode::getName() {
  return &myName[0];
}

char * CWirelessAPNode::getMACAddr() {
  return &myMac[0];
}

short CWirelessAPNode::getSignal() {
  return mySignal;
}

short CWirelessAPNode::getNoise() {
  return myNoise;
}

short CWirelessAPNode::getChannel() {
  return myChannel;
}

CWirelessAPNode * CWirelessAPNode::getSibling() {
  return mySibling;
}

void CWirelessAPNode::FreeMem() {
  if(mySibling!=NULL) {
    mySibling->FreeMem();
	delete mySibling;
  }
  mySibling = NULL;
}