#ifndef __CWIRELESSSAMPLE__
#define __CWIRELESSSAMPLE__

#ifndef NULL
#define NULL 0x00000000
#endif

// network communication structs
struct Apnodestruct {
  char mac[12];
  int channel;
  int signal;
  int noise;
};

struct MsgHeader {
  int proto;
  int type;
  int length;
};

struct MsgFromServer{
  struct MsgHeader header;
  int x;
  int y;
};

struct MsgToServer {
  struct MsgHeader header;
  struct Apnodestruct apnodes[16];
};
// end network communication structs

class CWirelessAPNode {
public:
  // constructors
  CWirelessAPNode(CWirelessAPNode * sibling);
  CWirelessAPNode(char * newName,
				  char * newMAC,
				  short newSignal,
				  short newNoise,
				  short newChannel,
				  CWirelessAPNode * sibling);
  CWirelessAPNode();

  // memory control, call FreeMem() before delete
  void FreeMem();

  // access functions
  char * getName();
  char * getMACAddr();
  short getSignal();
  short getNoise();
  short getChannel();
  CWirelessAPNode * getSibling();

  // manipulation functions
  void setName(char * newName);
  void setMACAddr(char * newMac);
  void setSignal(short newSignal);
  void setNoise(short newNoise);
  void setChannel(short newChannel);
  void setSibling(CWirelessAPNode * newSibling);

private:
  // private data members
  char myName[32];
  char myMac[16];
  short mySignal;
  short myNoise;
  short myChannel;

  // private data members
  CWirelessAPNode * mySibling;

};

class CWirelessSample {
public:
  // constructors
  CWirelessSample(CWirelessAPNode * newKid);
  CWirelessSample(MsgToServer * msg); // construct from a server message
  CWirelessSample();

  // memory control, call FreeMem() before
  // deleting
  // so to remove dynamically allocated memory do:
  // x->FreeMem(); delete x;
  void FreeMem();

  // access...like a linked list really
  // returns true iff at end, ie next ap name is NULL
  bool AtEnd();

  // increments current AP to next child, if !AtEnd()
  void Next();

  // resets current pointer to start
  void Reset();

  // returns a pointer to current AP
  CWirelessAPNode * Current();

  void SetChild(CWirelessAPNode * newKid);

  // generate a message describing the sample
  void GenerateMessage(MsgToServer * msg);

private:
  // private data members
  CWirelessAPNode * myKid;
  CWirelessAPNode * current;
};

#endif

