#include <afxdlgs.h>
#include <afxcmn.h>
#include <process.h>
#include "RFWindow.h"
#include "resource.h"
#include "ServerDialog.h"
#include "LocalDialog.h"
#include "RunDialog.h"

#include <math.h>

MsgToServer fromMe;
MsgFromServer toMe;

bool interrupted;
bool received;

#define LINE_MAC 0
#define LINE_X 1
#define LINE_Y 2
#define LINE_CHAN 3
#define LINE_SIG 4
#define LINE_NOISE 5

void DieWithError(char * msg) {
	AfxMessageBox(msg);
	_endthread();
}

void DoWait(void * seconds) {
	int time = (int) seconds;

	Sleep(1000*time);
	interrupted = true;
}

void DoSingleServer(void * wnd) {
	RFWindow * rfwnd = (RFWindow *) wnd;

	if (WSAStartup(MAKEWORD(2, 0), &rfwnd->wsaData) != 0) /* Load Winsock 2.0 DLL */
    {
        DieWithError("WSAStartup failed");
    }

    /* Create a best-effort datagram socket using UDP */
    if ((rfwnd->sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
        DieWithError("socket() failed");

    /* Construct the server address structure */
    memset(&rfwnd->echoServAddr, 0, sizeof(rfwnd->echoServAddr));    /* Zero out structure */
    rfwnd->echoServAddr.sin_family = AF_INET;                 /* Internet address family */
    rfwnd->echoServAddr.sin_addr.s_addr = htonl(rfwnd->servIP);  /* Server IP address */
	rfwnd->echoServAddr.sin_port   = htons(rfwnd->port);     /* Server port */

	// take the sample
	CWirelessSample * sample = rfwnd->GetAveragedSample();
	sample->GenerateMessage(&fromMe);

    /* Send the string, including the null terminator, to the server */
    if (sendto(rfwnd->sock, (char *) &fromMe, sizeof(fromMe), 0, (struct sockaddr *)
               &rfwnd->echoServAddr, sizeof(rfwnd->echoServAddr)) <= 0)
        DieWithError("sendto() sent a different number of bytes than expected");
  
	// delete sample & message
	sample->FreeMem();
	delete sample;

    /* Recv a response */

    rfwnd->fromSize = sizeof(rfwnd->fromAddr);
    if ((rfwnd->respStringLen = recvfrom(rfwnd->sock, (char *) &toMe, sizeof(toMe), 0, (struct sockaddr *) &rfwnd->fromAddr, 
                 (int *) &rfwnd->fromSize)) != sizeof(toMe))
        DieWithError("recvfrom() failed");

    if (rfwnd->echoServAddr.sin_addr.s_addr != rfwnd->fromAddr.sin_addr.s_addr)
    {
        DieWithError("Received packet from unknown location");
    }

    rfwnd->moveLocation(ntohl(toMe.x),ntohl(toMe.y));

    closesocket(rfwnd->sock);
    WSACleanup();  /* Cleanup Winsock */

	if(rfwnd->run_setting==RUN_ONESHOT)
		rfwnd->run_setting = RUN_DONTRUN;

}

void DoContinuousServer(void * wnd) {
	RFWindow * rfwnd = (RFWindow *) wnd;
	while(rfwnd->run_setting==RUN_CONTINUOUS) {
		DoSingleServer(wnd);
		Sleep(1000*rfwnd->nTimeBetweenRuns);
	}
}

void DoSingleHybrid(void * wnd) {
	RFWindow * rfwnd = (RFWindow *) wnd;
	CWirelessSample * sample;
	CPoint point, oldloc;

	if(rfwnd->run_setting==RUN_ONESHOT) { // needed for testing only
		srand(time(NULL));
	}

	sample = rfwnd->GetAveragedSample();
	if(rfwnd->dump_sample) rfwnd->dumpSample(sample);
	if(!rfwnd->rfmap->GetPosition(&point,1,sample)) { // do interpolation
		//AfxMessageBox("GetPosition() failed");
		//rfwnd->run_setting = RUN_DONTRUN;
		
		// Make no changes
		if(rfwnd->run_setting==RUN_ONESHOT)
			rfwnd->run_setting = RUN_DONTRUN;
	}

	oldloc.x = rfwnd->myLocation.x;
	oldloc.y = rfwnd->myLocation.y;

	rfwnd->moveLocation(point.x,point.y);

	int result = AfxMessageBox("Am I here?",MB_YESNOCANCEL);

	if(result==IDYES) {
		rfwnd->rfmap->AddSample(point.x,point.y,sample);
	} else {
		rfwnd->moveLocation(oldloc.x,oldloc.y);
		sample->FreeMem();
		delete sample;
	}

	if(result==IDCANCEL) rfwnd->run_setting = RUN_DONTRUN;

	if(rfwnd->run_setting==RUN_ONESHOT)
		rfwnd->run_setting = RUN_DONTRUN;

}

void DoContinuousHybrid(void * wnd) {
	RFWindow * rfwnd = (RFWindow *) wnd;
	while(rfwnd->run_setting==RUN_CONTINUOUS) {
		DoSingleHybrid(wnd);
		Sleep(1000*rfwnd->nTimeBetweenRuns);
	}
}

void DoSingleLocalLocate(void * wnd) {

	CPoint point;
	RFWindow * rfwnd = (RFWindow *) wnd;
	CWirelessSample * sample;
	
	if(rfwnd->run_setting==RUN_ONESHOT) { // needed for testing only
		srand(time(NULL));
	}

	sample = rfwnd->GetAveragedSample();
	if(rfwnd->dump_sample) rfwnd->dumpSample(sample);
	if(!rfwnd->rfmap->GetPosition(&point,rfwnd->getpos_mode,sample)) {
		//AfxMessageBox("GetPosition() failed");
		//rfwnd->run_setting = RUN_DONTRUN;
		// Do nothing
	}

	sample->FreeMem();
	delete sample;

	rfwnd->moveLocation(point.x,point.y);

	if(rfwnd->run_setting==RUN_ONESHOT)
		rfwnd->run_setting = RUN_DONTRUN;
}

void DoContinuousLocalLocate(void * wnd) {
	RFWindow * rfwnd = (RFWindow *) wnd;
	while(rfwnd->run_setting==RUN_CONTINUOUS) {
		DoSingleLocalLocate(wnd);
		Sleep(1000*rfwnd->nTimeBetweenRuns);
	}
}

RFWindow::RFWindow() {
	if(Create(NULL,TITLE_STRING,WS_OVERLAPPEDWINDOW,
		CRect(0,0,521,744),NULL,"IDR_MENU1p")==FALSE) {

		AfxMessageBox("Failed to create window");
		exit(-1);
	}
	
	if(bmp_map.LoadBitmap("map")==0) {
		AfxMessageBox("Failed to load map bitmap");
		exit(-1);
	}
	CClientDC dc(this);
	if(memDC.CreateCompatibleDC(&dc)==0) {
		AfxMessageBox("Failed to create compatible device context");
		exit(-1);
	}
	memDC.SelectObject(&bmp_map);
	
	if(bmp_x.LoadBitmap("redx")==0) {
		AfxMessageBox("Failed to load map bitmap");
		exit(-1);
	}
	if(xDC.CreateCompatibleDC(&dc)==0) {
		AfxMessageBox("Failed to create compatible device context");
		exit(-1);
	}
	xDC.SelectObject(&bmp_x);

	if(bmp_megerian.LoadBitmap("megerian")==0) {
		AfxMessageBox("Failed to load Megerian's picture");
		exit(-1);
	}
	if(megerianDC.CreateCompatibleDC(&dc)==0) {
		AfxMessageBox("Failed to create compatible device context");
		exit(-1);
	}
	megerianDC.SelectObject(&bmp_megerian);

	// generate the map itself
	rfmap = new CRFMap(521,696);
	currentFileName = "";
	RF_in_memory = false;
	dump_sample = false;

	// generate and setup the snooper
	snooper = new CSnooper();
	if(!snooper->Capture()) {
		AfxMessageBox("Unable to capture target window: Is the HP Site Manager running?");
		exit(-1);
	}

	mode_setting = MODE_NOMODE;
	run_setting = RUN_DONTRUN;

	nSamples = 0;
	nTimeBetweenSamples = 0;
	nTimeBetweenRuns = 0;

	draw_location = false;

	sock = -1;

	xpos = ypos = 0;

	// init for printing points
	x[0] = y[0] = -1;

	// record init
	recordFile = NULL;

}

RFWindow::~RFWindow() {
	
}

afx_msg void RFWindow::OnDestroy() {
	if(sock!=-1) closesocket(sock);
	if(recordFile!=NULL) {
		fclose(recordFile);
		recordFile = NULL;
	}
	delete rfmap;
	delete snooper;
	exit(0);
}

afx_msg void RFWindow::OnQuit() {
	SendMessage(WM_CLOSE);
}

afx_msg void RFWindow::OnLoad() {
	mode_setting = MODE_NOMODE;
	run_setting = RUN_DONTRUN;
	
	CFileDialog fd(TRUE,			// TRUE for open, FALSE for save as
				   NULL,		// Default extension
				   NULL,			// Initial file name
				   OFN_HIDEREADONLY | OFN_FILEMUSTEXIST,				// Flag values
				   "RF Map Files (*.rfmap)|All Files (*.*)|*.*||",
				   NULL);
	if(fd.DoModal()==IDCANCEL) return;

	currentFileName = fd.GetFileName();

	if(!rfmap->SetCurrentFile((char *) ((LPCSTR)currentFileName))) {
		AfxMessageBox("Unable to set file name in RFMap object");
		return;
	}
	
	if(!rfmap->ReadFromFile()) {
		AfxMessageBox("Unable to read from file:"+currentFileName);
		return;
	}

	// ** START OF ADDITION FOR POINT PRINTING ***

	FILE * infile = fopen(currentFileName,"r");
	if(infile==NULL) return;

	char buff_in[256];
	char * token;
	int line_type = LINE_MAC;
	int current = 0;
	while(fgets(buff_in,256,infile)&&current<1023) {
		token = strtok(buff_in," \t\n");
		switch(line_type) {
		case LINE_MAC:
			line_type = LINE_X;
			break;
		case LINE_X:
			x[current] = atoi(token);
			if(x[current]==-1) line_type = LINE_MAC;
			else line_type = LINE_Y;
			break;
		case LINE_Y:
			y[current] = atoi(token);
			line_type = LINE_SIG;
			current++;
			break;
		case LINE_SIG:
			line_type = LINE_NOISE;
			break;
		case LINE_NOISE:
			line_type = LINE_X;
			break;
		}
	}

	x[current] = y[current] = -1;

	fclose(infile);

	InvalidateRect(NULL);

	// ** END OF ADDITION FOR POINT PRINTING**

	AfxMessageBox("Loaded successfully from file: "+currentFileName);

	RF_in_memory = true;
}

afx_msg void RFWindow::OnNewRFMap() {
	mode_setting = MODE_NOMODE;
	run_setting = RUN_DONTRUN;
	delete rfmap;
	rfmap = new CRFMap(521,696);
	currentFileName = "";
	RF_in_memory = true;
}

afx_msg void RFWindow::OnSave() {
	if(!RF_in_memory) {
	  AfxMessageBox("No RF Map has been loaded or generated.");
	  return;
	}
	if(currentFileName=="") {
	  AfxMessageBox("No name has been specified for the RF Map: Use Save As");
	  return;
	}
	if(!rfmap->WriteToFile()) {
		AfxMessageBox("Unable to write to file: "+currentFileName);
		return;
	} else {
		AfxMessageBox("Wrote RF Map to file: "+currentFileName);
		return;
	}
}

afx_msg void RFWindow::OnSaveAs() {
	if(!RF_in_memory) {
	  AfxMessageBox("No RF Map has been loaded or generated.");
	  return;
	}
	
	CFileDialog fd(FALSE,			// TRUE for open, FALSE for save as
				   NULL,		// Default extension
				   NULL,			// Initial file name
				   OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,				// Flag values
				   "RF Map Files (*.rfmap)|All Files (*.*)|*.*||",
				   NULL);
	if(fd.DoModal()==IDCANCEL) return;

	currentFileName = fd.GetFileName();

	if(!rfmap->SetCurrentFile((char *)((LPCSTR) currentFileName))) {
	  AfxMessageBox("Unable to set file name in RFMap object");
	  currentFileName = "";
	  return;
	}

	OnSave();
}

afx_msg void RFWindow::OnModeServerLocate() {
	run_setting = RUN_DONTRUN;
	ServerDialog sd;
	if(sd.DoModal()==-1) {
		// cancelled, do nothing
		return;
	}
	// implement the rest here
	servIP = sd.ipaddr;
	port = sd.port;
	timeout = sd.timeout;
	dump_sample = sd.dump;
	
	nSamples = sd.nSamples;
	nTimeBetweenSamples = sd.nInterval;

	mode_setting = MODE_SERVER_LOCATE;
}

afx_msg void RFWindow::OnModeLocalMap() {
	run_setting = RUN_DONTRUN;
	LocalDialog ld;
	if(ld.DoModal()==-1) {
		// cancelled, do nothing
		return;
	}
	// implement the rest here

	nSamples = ld.nSamples;
	nTimeBetweenSamples = ld.nInterval;
	dump_sample = ld.dump;

	if(nSamples<=0) {
		AfxMessageBox("Unable to enter mapping mode: Number of samples <= 0");
		return;
	}

	if(nTimeBetweenSamples<=0) {
		AfxMessageBox("Unable to enter mapping mode: Time between samples <= 0");
		return;
	}

	mode_setting = MODE_LOCAL_MAP;
}

afx_msg void RFWindow::OnModeLocalLocate() {
	run_setting = RUN_DONTRUN;
	LocalDialog ld;
	if(ld.DoModal()==-1) {
		// cancelled, do nothing
		return;
	}
	// implement the rest here

	nSamples = ld.nSamples;
	nTimeBetweenSamples = ld.nInterval;
	dump_sample = ld.dump;

	mode_setting = MODE_LOCAL_LOCATE;

	
}

afx_msg void RFWindow::OnModeLocalHybrid() {
	run_setting = RUN_DONTRUN;
	LocalDialog ld;
	if(ld.DoModal()==-1) {
		// cancelled, do nothing
		return;
	}
	// implement the rest here
	nSamples = ld.nSamples;
	nTimeBetweenSamples = ld.nInterval;
	dump_sample = ld.dump;

	mode_setting = MODE_LOCAL_HYBRID;
}

afx_msg void RFWindow::OnRunContinuous() {
	run_setting = RUN_DONTRUN;
	if(mode_setting==MODE_LOCAL_MAP) {
		AfxMessageBox("Can't start running: already in mapping mode.");
		return;
	} else if(mode_setting==MODE_NOMODE) {
		AfxMessageBox("Can't start running: no mode selected.");
		return;
	}
	RunDialog rd;
	rd.nInterval = nTimeBetweenRuns;
	rd.nSamples = nSamples;
	rd.mode = getpos_mode;
	if(rd.DoModal()==-1) {
		return;
	}
	nTimeBetweenRuns = rd.nRunDelta;
	getpos_mode = rd.mode;

	if(!RF_in_memory) {
		AfxMessageBox("Can't run--No RF in memory");
		return;
	}

	// do the rest here
	run_setting = RUN_CONTINUOUS;
	if(mode_setting == MODE_LOCAL_LOCATE) {
		AfxMessageBox("Starting Local/Locate Mode Continuously");
		_beginthread(DoContinuousLocalLocate,0,(void *) this);
	} else if (mode_setting == MODE_SERVER_LOCATE) {
		AfxMessageBox("Starting Server/Locate Mode Continuously");
		// do server stuff

			_beginthread(DoContinuousServer,0,(void *) this);

	} else {
		AfxMessageBox("Starting Local/Hybrid Mode Continuously");
		// do local hybrid stuff
		_beginthread(DoContinuousHybrid,0,(void *) this);
	}
}

afx_msg void RFWindow::OnRunOneShot() {
	run_setting = RUN_DONTRUN;
	if(mode_setting==MODE_LOCAL_MAP) {
		AfxMessageBox("Can't start running: already in mapping mode.");
		return;
	} else if(mode_setting==MODE_NOMODE) {
		AfxMessageBox("Can't start running: no mode selected.");
		return;
	}
	nTimeBetweenRuns = 0;

	RunDialog rd;
	rd.nInterval = nTimeBetweenRuns;
	rd.nSamples = nSamples;
	rd.nRunDelta = -10;
	rd.mode = getpos_mode;
	
	if(rd.DoModal()==-1) {
		return;
	}
	getpos_mode = rd.mode;

	if(!RF_in_memory) {
		AfxMessageBox("Can't run--No RF in memory");
		return;
	}

	// do the rest here
	run_setting = RUN_ONESHOT;
	if(mode_setting == MODE_LOCAL_LOCATE) {
		_beginthread(DoSingleLocalLocate,0,(void *) this);
	} else if (mode_setting == MODE_SERVER_LOCATE) {
		// do server stuff

		_beginthread(DoSingleServer,0,(void *) this);

	} else {
		// do local hybrid stuff
		_beginthread(DoSingleHybrid,0,(void *) this);
	}
}

afx_msg void RFWindow::OnRunStopRunning() {
	run_setting = RUN_DONTRUN;
}

afx_msg void RFWindow::OnHelpMe() {
	AfxMessageBox("Go find Jake or Dan!");
}


afx_msg void RFWindow::OnPaint() {
	CPaintDC dc(this);
	CPen r_pen;
	r_pen.CreatePen(PS_SOLID,1,RGB(255,0,0));
	dc.SelectObject(&r_pen);

	// draw the map itself
	dc.BitBlt( 0,0,521,696,&memDC,0,0,SRCCOPY);

	if(draw_location) {
		dc.BitBlt( myLocation.x-3,myLocation.y-2,5,6,&xDC,0,0,SRCCOPY);

		dc.MoveTo(oldLocation.x,oldLocation.y);
		dc.LineTo(myLocation.x,myLocation.y);

		if( myLocation.x > 149 && myLocation.x < 194 &&
			myLocation.y > 250 && myLocation.y < 343) { // near megerian's office
			dc.BitBlt(10,10,150,200,&megerianDC,0,0,SRCCOPY);
			dc.TextOut(10,210,"\"Our fearless leader\"");
		}
	}

	// code for printing distance
	char buff[12];
	int delta_x = xpos - myLocation.x;
	int delta_y = ypos - myLocation.y;
	double distance = sqrt((double) delta_x*delta_x + delta_y*delta_y);
	gcvt(distance,9,buff);
	dc.TextOut(0,0,buff);

	// added code for point coloring
	int current = 0;
	CPen g_pen;
	g_pen.CreatePen(PS_SOLID,2,RGB(0,128,0));
	dc.SelectObject(&g_pen);
	while(x[current]!=-1) {
		dc.MoveTo(x[current]-1,y[current]-1);
		dc.LineTo(x[current]+1,y[current]+1);
		dc.MoveTo(x[current]+1,y[current]-1);
		dc.LineTo(x[current]-1,y[current]+1);
		current++;
	}

}  

afx_msg void RFWindow::OnLButtonDown(UINT nFlags, CPoint point) {
	CRect rect;
	CWirelessSample * sample = NULL;
	
	if(!(nFlags & MK_RBUTTON)) {
		xpos = point.x;
		ypos = point.y ;
		InvalidateRect(NULL);
		return;
	}

	if(mode_setting==MODE_LOCAL_MAP) {
		GetWindowRect(&rect);
		// take a sample and add it to the RF map
		sample = GetAveragedSample();
		
		// dump the sample!!!!!!
		//	AfxMessageBox("reset done");
		if(dump_sample) {
			dumpSample(sample);
		}

		if(!rfmap->AddSample(point.x,point.y,sample)){
			AfxMessageBox("Attempt to add sample failed.");
		} else {
			AfxMessageBox("Sample added.");
			RF_in_memory = true;
			moveLocation(point.x,point.y);
			
		}

	} else {
		AfxMessageBox("Can't take a sample--Not in mapping mode");
	}

}

afx_msg void RFWindow::OnRButtonDown(UINT nFlags, CPoint point) {

/*
	CString temp = "Right click: (";
	char buff[8];
	itoa(point.x,buff,10);
	temp+=buff;
	itoa(point.y,buff,10);
	temp+=",";
	temp+=buff;
	temp+=")";
	AfxMessageBox(temp); */
}

afx_msg void RFWindow::OnMButtonDown(UINT nFlags, CPoint point) {
	
	if(AfxMessageBox("Undo last add to the RF Map?",MB_YESNO)==IDNO) return;
	if(rfmap->UndoLastAdd()) {
		AfxMessageBox("Undo was successful.");
	} else {
		AfxMessageBox("Unable to undo last add.");
	}
}

CWirelessSample * RFWindow::GetAveragedSample() {
	CWirelessSample * sample, * sample1;
	CWirelessAPNode * apnode, * apnode1;

	if(nSamples<1) return NULL;

	bool found;
	SetCursor(LoadCursor(NULL,IDC_WAIT));
	
	// get first sample 
	sample = snooper->Sample();

	for(int i=1;i<nSamples;i++) {
		Sleep(1000*nTimeBetweenSamples);

		// take next sample
		sample1 = snooper->Sample();

		// reset the samples for the combination loop
		sample1->Reset();

		while(!sample1->AtEnd()) {
			apnode1 = sample1->Current();

			sample->Reset();
			found = false;
			while(!found && !sample->AtEnd()) {
				apnode = sample->Current();
				if(strcmp(apnode->getMACAddr(),apnode1->getMACAddr())==0) {
					apnode->setSignal(apnode->getSignal()+apnode1->getSignal());
					apnode->setNoise(apnode->getNoise()+apnode1->getNoise());
					found = true;
				}
				sample->Next();
			}

			sample1->Next();
		}

		sample1->FreeMem();
		delete sample1;

	}

	// divide out the averages
	sample->Reset();
	while(!sample->AtEnd()) {
		apnode = sample->Current();
		apnode->setSignal(apnode->getSignal() / nSamples);
		apnode->setNoise(apnode->getNoise() / nSamples);
		sample->Next();
	}

	sample->Reset();
	
	SetCursor(LoadCursor(NULL,IDC_ARROW));
	return sample;
//	sample_lock = false;

	// temporary implementation that does not implement averaging
//	return snooper->Sample();
	
}

void RFWindow::moveLocation(int new_x, int new_y) {

	oldLocation.x = myLocation.x;
	oldLocation.y = myLocation.y;
	myLocation.x = new_x;
	myLocation.y = new_y;
	draw_location= true;
	Invalidate(TRUE);
	record();

}

void RFWindow::dumpSample(CWirelessSample * sample) {
	if(sample==NULL) return;
	CWirelessAPNode * apnode;

	CString temp; 
	temp = "Beginning dump of sample:\n";
	char buff[8];

	sample->Reset();

	while(!sample->AtEnd()) {
		apnode = sample->Current();
	//	AfxMessageBox("Got apnode");
		temp+="MAC=";
		temp+=apnode->getMACAddr();
	//	AfxMessageBox("Got mac addr");
		temp+="  SIG:";
		itoa(apnode->getSignal(),buff,10);
		temp+=buff;
		temp+="  NOISE:";
		itoa(apnode->getNoise(),buff,10);
	//	AfxMessageBox("Got noise");
		temp+=buff;
		temp+="\n";
		sample->Next();
	}
	sample->Reset();
	AfxMessageBox(temp); 
}

afx_msg void RFWindow::OnRecord() {


	CFileDialog fd(FALSE,			// TRUE for open, FALSE for save as
				   NULL,		// Default extension
				   NULL,			// Initial file name
				   OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,				// Flag values
				   "All Files (*.*)|*.*||",
				   NULL);
	if(fd.DoModal()==IDCANCEL) return;

	if(recordFile!=NULL) {
		fclose(recordFile);
		recordFile = NULL;
	}

	recordFile = fopen(fd.GetFileName(),"w");
	if(recordFile==NULL) {
		AfxMessageBox("Unable to open specified file!");
		return;
	}

	AfxMessageBox("Now recording locating data.\n");

	
}

void RFWindow::record() {
	if(recordFile==NULL) return;

	// else write to the record!
	// format is:
	// True x, True y, Est x, Est y, distance
	char buff[1024];
	double distance;
	distance = sqrt( (myLocation.x-xpos)*(myLocation.x-xpos) +
					 (myLocation.y-ypos)*(myLocation.y-ypos) );
	sprintf(buff,"%i,%i,%i,%i,%2.10f\n",xpos,ypos,
			myLocation.x,myLocation.y,distance);
	fputs(buff,recordFile);
	fflush(recordFile);

}

// hotkeys:
// L	Load
// r	Run One-shot
// R	Run Continuosly
// l	Local/Locate Mode
// m	Local/Map mode

afx_msg void RFWindow::OnChar(UINT uChar, UINT nRepCnt,UINT nFlags) {
	switch(uChar) {
	case 'r':
		OnRunOneShot();
		break;
	case 'R':
		OnRunContinuous();
		break;
	case 'L':
		OnLoad();
		break;
	case 'l':
		OnModeLocalLocate();
		break;
	case 'm':
		OnModeLocalMap();
		break;
	default:
		break;
	}
}

BEGIN_MESSAGE_MAP( RFWindow, CFrameWnd )
  ON_COMMAND(ID_FILE_NEW,OnNewRFMap)
  ON_COMMAND(ID_FILE_LOAD,OnLoad)
  ON_COMMAND(ID_FILE_SAVE,OnSave)
  ON_COMMAND(ID_FILE_SAVEAS,OnSaveAs)
  ON_COMMAND(ID_FILE_QUIT,OnQuit)

  ON_COMMAND(ID_MODE_SERVER_LOCATE,OnModeServerLocate)
  ON_COMMAND(ID_MODE_LOCAL_MAP,OnModeLocalMap)
  ON_COMMAND(ID_MODE_LOCAL_LOCATE,OnModeLocalLocate)
  ON_COMMAND(ID_MODE_LOCAL_HYBRID,OnModeLocalHybrid)

  ON_COMMAND(ID_RUN_CONTINUOUS,OnRunContinuous)
  ON_COMMAND(ID_RUN_ONESHOT,OnRunOneShot)
  ON_COMMAND(ID_RUN_STOPRUNNING,OnRunStopRunning)

  ON_COMMAND(ID_RECORD,OnRecord)

  ON_COMMAND(ID_HELP_HELPME,OnHelpMe)

  ON_WM_PAINT()
  ON_WM_DESTROY()

  ON_WM_LBUTTONDOWN()
  ON_WM_RBUTTONDOWN()
  ON_WM_MBUTTONDOWN()

  ON_WM_CHAR()

END_MESSAGE_MAP()

