#include <iostream>
#include <string.h>
#include "mactree.h"
using namespace std;

macTree::macTree(int x, int y, char *mac, int s, int n, int c) {
  root = new macTreeNode(x, y, mac, s, n, c);
  current = root;
}

macTree::macTree(int x, int y, char *mac, int s, int n, int c, int count) {
  root = new macTreeNode(x, y, mac, s, n, c, count);
  current = root;
}

macTree::~macTree() {
  freeTree(root);
}

void macTree::freeTree(macTreeNode *next) {
  if(next != NULL) {
    freeTree(next->getSmaller());
    freeTree(next->getLarger());
    delete next;
  }
}

void macTree::addMac(int x, int y, char *mac, int s, int n, int c) {
  macTreeNode *temp = root;
  macTreeNode *next;
  int done = 0;
  int scmp;
  while(!done) {
    scmp = strcmp(mac, temp->getMac());
    if(scmp < 0) {
      next = temp->getSmaller();
      if(next == NULL) {
	macTreeNode *small = new macTreeNode(x, y, mac, s, n, c);
	temp->setSmaller(small);
	done = 1;
      }
      else {
	temp = next;
      }
    }
    else if(scmp > 0){
      next = temp->getLarger();
      if(next == NULL) {
	macTreeNode *large = new macTreeNode(x, y, mac, s, n, c);
	temp->setLarger(large);
	done = 1;
      }
      else {
	temp = next;
      }
    }
    else {
      temp->addPos(x, y, s, n, c);
      done = 1;
    }
  }
}

void macTree::addMac(int x, int y, char *mac, int s, int n, int c, int count) {
  macTreeNode *temp = root;
  macTreeNode *next;
  int done = 0;
  int scmp;
  while(!done) {
    scmp = strcmp(mac, temp->getMac());
    if(scmp < 0) {
      next = temp->getSmaller();
      if(next == NULL) {
	macTreeNode *small = new macTreeNode(x, y, mac, s, n, c, count);
	temp->setSmaller(small);
	done = 1;
      }
      else {
	temp = next;
      }
    }
    else if(scmp > 0){
      next = temp->getLarger();
      if(next == NULL) {
	macTreeNode *large = new macTreeNode(x, y, mac, s, n, c, count);
	temp->setLarger(large);
	done = 1;
      }
      else {
	temp = next;
      }
    }
    else {
      temp->addPos(x, y, s, n, c, count);
      done = 1;
    }
  }
}

char *macTree::getMac() {
  return current->getMac();
}

bool macTree::moveSmaller() {
  macTreeNode *temp = current->getSmaller();
  if(temp == NULL) {
    return false;
  }
  current = temp;
  return true;
}

bool macTree::moveLarger() {
  macTreeNode *temp = current->getLarger();
  if(temp == NULL) {
    return false;
  }
  current = temp;
  return true;
}

bool macTree::setCurrent(char *mac) {
  macTreeNode *temp = root;
  int scmp;
  while(true) {
    scmp = strcmp(mac, temp->getMac());
    if(scmp == 0) {
      current = temp;
      return true;
    }
    if(scmp < 0) {
      temp = temp->getSmaller();
    }
    else {
      temp = temp->getLarger();
    }
    if(temp == NULL) {
      return false;
    }
  }
  return false; //never hit
}

positionLL *macTree::getPos() {
  return current->getPos();
}

positionLL *macTree::getMacPos(char *mac) {
  macTreeNode *temp = current;
  if(setCurrent(mac)) {
    macTreeNode *temp2;
    temp2 = current;
    current = temp;
    return temp2->getPos();
  }
  return NULL;
}

void macTree::writeToFile(ofstream *out) {
  root->write(out);
}

macTreeNode::macTreeNode(int x, int y, char *mac, int s, int n, int c) {
  myMac = new char[18];
  strcpy(myMac, mac);
  myPositions = new positionLL(x, y, s, n, c);
  smaller = NULL;
  larger = NULL;
}

macTreeNode::macTreeNode(int x, int y, char *mac, int s, int n, int c, int count) {
  myMac = new char[18];
  strcpy(myMac, mac);
  myPositions = new positionLL(x, y, s, n, c, count);
  smaller = NULL;
  larger = NULL;
}

macTreeNode::~macTreeNode() {
  delete myMac;
  delete myPositions;
}

void macTreeNode::addPos(int x, int y, int s, int n, int c) {
  myPositions->addPos(x, y, s, n, c);
}

void macTreeNode::addPos(int x, int y, int s, int n, int c, int count) {
  myPositions->addPos(x, y, s, n, c, count);
}

void macTreeNode::setSmaller(macTreeNode *newnode) {  smaller = newnode;
}

void macTreeNode::setLarger(macTreeNode *newnode) {
  larger = newnode;
}

void macTreeNode::setMac(char *mac) {
  strcpy(myMac, mac);
}

char *macTreeNode::getMac() {
  return myMac;
}

macTreeNode *macTreeNode::getSmaller() {
  return smaller;
}

macTreeNode *macTreeNode::getLarger() {
  return larger;
}

positionLL *macTreeNode::getPos() {
  return myPositions;
}

void macTreeNode::write(ofstream *out) {
  
  *out << myMac << endl;
  
  myPositions->reset();
  do {
    *out << myPositions->getx() << endl;
    *out << myPositions->gety() << endl;
    *out << myPositions->getCount() << endl;
    *out << myPositions->getSignal() << endl;
    *out << myPositions->getNoise() << endl;
    *out << myPositions->getChannel() << endl;
  }while(myPositions->next());
  *out << "-1\n"; //signifies new mac node  

  if(smaller != NULL) {
    smaller->write(out);
  }
  if(larger != NULL) {
    larger->write(out);
  }
}

positionLL::positionLL(int newx, int newy, int news, int newn, int newc) {
  head = new positionNode(newx, newy, news, newn, newc);
  current = head;
  end = head;
}

positionLL::positionLL(int newx, int newy, int news, int newn, int newc, int count) {
  head = new positionNode(newx, newy, news, newn, newc, count);
  current = head;
  end = head;
}

positionLL::~positionLL() {
  current = head;
  positionNode *temp;
  while(current != end) {
    temp = current;
    current = temp->getNext();
    delete temp;
  }
  delete current;
}

int *positionLL::addPos(int x, int y, int s, int n, int c) {
  end->setNext(new positionNode(x, y, s, n, c));
  end = end->getNext();
  return end->getCountPoint();
}

int *positionLL::addPos(int x, int y, int s, int n, int c, int count) {
  end->setNext(new positionNode(x, y, s, n, c, count));
  end = end->getNext();
  return end->getCountPoint();
}

void positionLL::getPos(int *x, int *y) {
  current->getPosition(x, y);
}

bool positionLL::next() {
  if(current == end) return false;
  current = current->getNext();
  return true;
}

bool positionLL::atEnd() {
  if(current == end) return true;
  return false;
}

void positionLL::reset() {
  current = head;
}

void positionLL::setMax() {
  positionNode *maxNode;
  maxNode = current;
  int max = current->getCount();
  while(next()) {
    if(max < current->getCount()) {
      max = current->getCount();
      maxNode = current;
    }
  }
  current = maxNode;
}

int *positionLL::getCurrentCount() {
  return current->getCountPoint();
}

int positionLL::getx() {
  return current->getx();
}

int positionLL::gety() {
  return current->gety();
}

int positionLL::getCount() {
  return current->getCount();
}

int positionLL::getSignal() {
  return current->getSignal();
}

int positionLL::getNoise() {
  return current->getNoise();
}

int positionLL::getChannel() {
  return current->getChannel();
}

positionNode::positionNode(int newx, int newy, int news, int newn, int newc) {
  x=newx;
  y=newy;
  count=1;
  nextPosition=NULL;
  signal = news;
  noise = newn;
  channel = newc;
}

positionNode::positionNode(int newx, int newy, int news, int newn, int newc, int newcount) {
  x=newx;
  y=newy;
  count=newcount;
  nextPosition=NULL;
  signal = news;
  noise = newn;
  channel = newc;
}

positionNode::~positionNode() {
}

void positionNode::setPosition(int newx, int newy) {
  x=newx;
  y=newy;
  count=1;
}

void positionNode::getPosition(int *posx, int *posy) {
  *posx=x;
  *posy=y;
}

void positionNode::setNext(positionNode *newNext) {
  nextPosition = newNext;
}

positionNode *positionNode::getNext() {
  return nextPosition;
}

int positionNode::getCount() {
  return count;
}

void positionNode::incCount() {
  count++;
}

int *positionNode::getCountPoint() {
  return &count;
}

int positionNode::getx() {
  return x;
}

int positionNode::gety() {
  return y;
}

int positionNode::getSignal() {
  return signal;
}

int positionNode::getNoise() {
  return noise;
}

int positionNode::getChannel() {
  return channel;
}
