#include <fstream>
using namespace std;

class positionNode {
 public:
  positionNode(int, int, int, int, int);
  positionNode(int, int, int, int, int, int);
  ~positionNode();
  void setPosition(int, int);
  void getPosition(int *, int *);
  void setNext(positionNode *);
  positionNode *getNext();
  int getCount();
  int *getCountPoint();
  void incCount();
  int getx();
  int gety();
  int getSignal();
  int getNoise();
  int getChannel();

 private:
  int x;
  int y;
  int count;
  int signal;
  int noise;
  int channel;
  positionNode *nextPosition;
};

class positionLL {
 public:
  //construct position x,y
  //signal, noise, channel
  positionLL(int, int, int, int, int);

  //construct x, y, signal, noise, channel, count
  positionLL(int, int, int, int, int, int);

  //destruct linked list
  ~positionLL();

  //add a position x,y,signal,noise,channel
  //returns a pointer to the count
  int *addPos(int, int, int, int, int);

  //add a position x, y, signal, noise, channel, count
  //returns a pointer to the count
  int *addPos(int, int, int, int, int, int);

  //returns false if at end
  //else moves to next and returns true
  bool next();

  //returns true if at end
  //false if not at end
  bool atEnd();

  //resets the position in the list
  void reset();

  //sets current to the node with max count starting with current
  //picks first if there are multiple largest
  void setMax();

  //returns a pointer the count of the current node
  int *getCurrentCount();

  //gets current position x,y
  void getPos(int *, int *);

  int getx();
  int gety();
  int getCount();
  int getSignal();
  int getNoise();
  int getChannel();

 private:
  positionNode *head;
  positionNode *end;
  positionNode *current;
};

class macTreeNode {
 public:
  //construct node x, y, mac, signal, noise, channel
  macTreeNode(int, int, char *, int, int, int);

  //construct node x, y, mac, signal, noise, channel, count
  macTreeNode(int, int, char *, int, int, int, int);

  //destruct node
  ~macTreeNode();

  //sets the smaller node
  void setSmaller(macTreeNode *);

  //sets the larger node
  void setLarger(macTreeNode *);

  //set my mac
  void setMac(char *);

  //get my mac
  char *getMac();

  //get smaller node
  macTreeNode *getSmaller();

  //get larger node
  macTreeNode *getLarger();

  //get my positions
  positionLL *getPos();

  //add position
  void addPos(int, int, int, int, int);

  //add position
  void addPos(int, int, int, int, int, int);

  //write to a file
  void write(ofstream *);

 private:
  char *myMac;
  positionLL *myPositions;
  macTreeNode *smaller;
  macTreeNode *larger;
};

class macTree {
 public:
  //construct tree, x, y, mac, signal, noise, channel
  macTree(int, int, char *, int, int, int);

  //construct tree, x, y, mac, signal, noise, channel, count
  macTree(int, int, char *, int, int, int, int);

  //destruct tree
  ~macTree();

  //add mac and position x, y, mac, signal, noise, channel
  void addMac(int, int, char *, int, int, int);

  //add mac and position x, y, mac, signal, noise, channel, count
  void addMac(int, int, char *, int, int, int, int);

  //get current mac
  char *getMac();

  //move current to smaller
  //returns true on success
  //returns false if there is no smaller
  bool moveSmaller();

  //move current to larger
  //returns true on success
  //returns false if there is no larger
  bool moveLarger();

  //sets current to the given mac
  //returns true if it is present
  //false if it is not
  bool setCurrent(char *);

  //returns position ll of current node
  positionLL *getPos();

  //returns position ll of a given mac
  //returns NULL if mac is not in tree
  positionLL *getMacPos(char *);

  //write to file
  void writeToFile(ofstream *);

 private:
  macTreeNode *root;
  macTreeNode *current;
  
  //deletes the tree from a given node downward
  void freeTree(macTreeNode *);
};

