// DUMMY CRFMAP FUNCTIONS
// DUMMY CRFMAP FUNCTIONS

#include <string.h>
#include <iostream>
#include <math.h>
#include "CRFMap.h"
using namespace std;

#ifndef NULL
#define NULL 0x00000000
#endif

CRFMap::CRFMap(int sizex, int sizey) {
  fileSet = false;
  myTree = NULL;
  xsize = sizex;
  ysize = sizey;
  int i, j;
  macCount = new int **[sizex];
  for(i=0; i<xsize; i++) {
    macCount[i] = new int *[sizey];
    for(j=0; j<ysize; j++) {
      macCount[i][j] = new int;
      *macCount[i][j] = 0;
    }
  }
  mapFile = NULL;
  undo = false;
  lastx = -1;
  lasty = -1;
}
CRFMap::~CRFMap() {
  int i, j;
  for(i=0; i<xsize; i++) {
    for(j=0; j<ysize; j++)
      delete macCount[i][j];
    delete[] macCount[i];
  }
  delete[] macCount;
  delete myTree;
}

bool CRFMap::SetCurrentFile(char * fileName) {
  if(fileName==NULL) return false;
  fileSet = true;
  if(mapFile == NULL) mapFile = new char[512];
  strcpy(mapFile, fileName);
  return true;
}

bool CRFMap::ReadFromFile() {
  if(fileSet) {
    ifstream in;
    in.open(mapFile);
    if(!in) {
      return false;
    }
    if(myTree != NULL) {
      delete myTree;
    }
    char *mac = new char[18];
    int x, y, signal, noise, channel, count;
    in >> mac >> x >> y >> count >> signal >> noise >> channel;
    myTree = new macTree(x, y, mac, signal, noise, channel, count);
    *macCount[x][y] = *macCount[x][y] + 1;
    in >> x;
    while(x > 0) {
      in >> y >> count >> signal >> noise >> channel;
      myTree->addMac(x, y, mac, signal, noise, channel, count);
      *macCount[x][y] = *macCount[x][y] + 1;
      in >> x;
    }
    in >> mac;
    while(!in.eof()) {
      in >> x;
      while(x >= 0) {
	in >> y >> count >> signal >> noise >> channel;
	myTree->addMac(x, y, mac, signal, noise, channel, count);
	*macCount[x][y] = *macCount[x][y] + 1;
	in >> x;
      }
      in >> mac;
    }
  }
  return fileSet;
}

bool CRFMap::AddSample(int x, int y, CWirelessSample * sample) {
  if(sample==NULL || x<0 || y<0) return false;
  sample->Reset();
  CWirelessAPNode *current = sample->Current();
  if(current == NULL) return false;
  if(myTree == NULL) {
    myTree = new macTree(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
  }
  else {
    myTree->addMac(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
  }
  sample->Next();
  while(!sample->AtEnd()) {
    current = sample->Current();
    myTree->addMac(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
    sample->Next();
  }
  sample->FreeMem();
  delete sample;
  undo = true;
  return true;
}

bool CRFMap::GetPosition(CPoint * point, int mode, CWirelessSample * sample) {
  if(point==NULL || sample == NULL) return false;
  switch(mode) {
  case 0: return getPos0(point, sample);
  case 1: return getPos1(point, sample);
  case 2: return getPos2(point, sample);
  case 3: return getPos3(point, sample);
  case 4: return getPos4(point, sample);
  default: return false;
  }
}

positionLL *CRFMap::getPosCounts(CPoint *point, CWirelessSample *sample) {
  //an array of counts for each position
  int ***count = new int **[xsize];
  int i, j;
  for(i=0; i<xsize; i++) {
    count[i] = new int *[ysize];
    for(j=0; j<ysize; j++) {
      count[i][j] = new int;
      *count[i][j] = 0;
    }
  }

  //Linked list of positions
  positionLL *posList = NULL;
  int x, y;

  sample->Reset();
  CWirelessAPNode *current;
  positionLL *currentPosLL;
  char *mac;

  while(!sample->AtEnd()) {
    current = sample->Current();
    mac = current->getMACAddr();
    currentPosLL = myTree->getMacPos(mac);
    if(currentPosLL != NULL) {
      currentPosLL->reset();
      currentPosLL->getPos(&x, &y);
      if(posList == NULL) {
	posList = new positionLL(x, y, currentPosLL->getSignal(), currentPosLL->getNoise(), currentPosLL->getChannel());
	count[x][y] = posList->getCurrentCount();
      }
      else {
	if(*count[x][y] == 0) {
	  delete count[x][y];
	  count[x][y] = posList->addPos(x, y, currentPosLL->getSignal(), currentPosLL->getNoise(), currentPosLL->getChannel());
	}
	else {
	  *count[x][y] = *count[x][y] + 1;
	}
      }
      while(currentPosLL->next()) {
	currentPosLL->getPos(&x, &y);
	if(*count[x][y] == 0) {
	  delete count[x][y];
	  count[x][y] = posList->addPos(x, y, currentPosLL->getSignal(), currentPosLL->getNoise(), currentPosLL->getChannel());
	}
	else *count[x][y] = *count[x][y] + 1;
      }
    }
    sample->Next();
  }
  for(i=0; i<xsize; i++) {
    for(j=0; j<ysize; j++)
      if(*count[i][j] == 0) //don't want to delete the ones in poslist
	delete count[i][j];
    delete[] count[i];
  }
  delete[] count;
  return posList;
}

positionLL *CRFMap::getMaxes(positionLL *posList) {
  if(posList == NULL) return NULL;
  posList->reset();
  posList->setMax();
  int maxCount = *posList->getCurrentCount();
  positionLL *newPosList = new positionLL(posList->getx(), posList->gety(), posList->getSignal(), posList->getNoise(), posList->getChannel());
  int done = 0;
  if(!posList->next()) {
    done = 1;
  }
  while(!done) {
    posList->setMax();
    if(maxCount != *posList->getCurrentCount()) {
      done = 1;
    }
    else {
      newPosList->addPos(posList->getx(), posList->gety(), posList->getSignal(), posList->getNoise(), posList->getChannel());
    }
    if(!posList->next()) {
      done = 1;
    }
  }
  return newPosList;
}

bool CRFMap::leastAPs(positionLL *newPosList) {
  if(newPosList == NULL) return false;
  
  int *temp;
  newPosList->reset();
  do {
    temp = newPosList->getCurrentCount();
    *temp = *temp-*macCount[newPosList->getx()][newPosList->gety()]; //added: *temp
    //*temp = -*macCount[newPosList->getx()][newPosList->gety()];
  }while(newPosList->next());
  return true;
}

bool CRFMap::weightSignal(positionLL *posList, CWirelessSample *sample) {
  posList->reset();
  char *sampleMac;
  int sampleSignal, currentSignal;
  CWirelessAPNode *current_sample;
  positionLL *treeLL;
  int posx, posy, treex, treey;
  int done, match;
  int *weight;

  sample->Reset();
  
  while(!sample->AtEnd()) {
    current_sample = sample->Current();
    sampleMac = current_sample->getMACAddr();
    sampleSignal = current_sample->getSignal();
    treeLL = myTree->getMacPos(sampleMac);
    if(treeLL != NULL) {
      posList->reset();
      do {
	posList->getPos(&posx, &posy);
	done = 0;
	match = 0;
	treeLL->reset();
	while(!done) {
	  treeLL->getPos(&treex, &treey);
	  if((treex == posx) & (treey == posy)) {
	    done = 1;
	    match = 1;
	  } else if(!treeLL->next()) done = 1;
	}
	if(match) {
	  weight = posList->getCurrentCount();
	  currentSignal = treeLL->getSignal();
	  *weight = *weight - abs(sampleSignal - currentSignal); ///////abs!!!!!
	}
      } while(posList->next()); 
    }
    sample->Next();
  }
  return true;
}

bool CRFMap::getPos3(CPoint *point, CWirelessSample *sample) {
  
  //weight the positions based on the number of APs matched
  positionLL *posList;
  posList = getPosCounts(point, sample);

  //pick out all the matching largest positions
  positionLL *newPosList;
  newPosList = getMaxes(posList);

  //weight the positions based on the number of APs in a position
  //less is weighted more
  if(!leastAPs(newPosList)) return false;

  //pick out all the matching positions with the least APs
  positionLL *newerPosList;
  newerPosList = getMaxes(newPosList);

  //weight the positions based on signal strength difference
  //between them and the sample
  if(!weightSignal(newerPosList, sample)) return false;

  //get position with highest signal weight
  newerPosList->reset();
  newerPosList->setMax();
  newerPosList->getPos((int *)&point->x, (int *)&point->y);

  delete newerPosList;
  delete newPosList;
  delete posList;
  return true;
}

bool CRFMap::getPos2(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);

  positionLL *newPosList;
  newPosList = getMaxes(posList);

  if(!leastAPs(newPosList)) return false;

  newPosList->reset();
  newPosList->setMax();
  newPosList->getPos((int *)&point->x, (int *)&point->y);

  delete newPosList;
  delete posList;
  return true;
}

bool CRFMap::getPos0(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);
  if(posList == NULL) return false;
  posList->reset();
  posList->setMax();
  posList->getPos((int *)&point->x, (int *)&point->y);

  delete posList;
  return true;
}

bool CRFMap::WriteToFile() {
  if(!fileSet) return false;
  ofstream out;
  out.open(mapFile);
  if(!out) {
    cout << "oops\n";
    return false;
  }
  myTree->writeToFile(&out);
  out.close();
  return true;
}

bool CRFMap::UndoLastAdd() {
  //if(undo) {
  //  undo = false;
  //  myTree->removeCurrent();
  //  return true;
  //}
  return false;
}

bool CRFMap::getPos1(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);
  if(posList == NULL) return false;
  if(!leastAPs(posList)) return false;
  addWeight(posList);
  if(!weightSignal(posList, sample)) return false;
  interpolate(posList, point);
  return true;
}

void CRFMap::addWeight(positionLL *posList) {
  //100 weight for now
  int *temp;
  posList->reset();
  do {
    temp = posList->getCurrentCount();
    *temp = *temp * 100;
  } while(posList->next());
}

void CRFMap::interpolate(positionLL *posList, CPoint *point) {
  int x1, x2, x3, y1, y2, y3, total, w1, w2, w3, x, y;
  int *temp;
  posList->reset();
  posList->setMax();
  temp = posList->getCurrentCount();
  x1 = posList->getx();
  y1 = posList->gety();
  w1 = *temp;
  *temp = 0x80000000;
  posList->reset();
  posList->setMax();
  temp = posList->getCurrentCount();
  x2 = posList->getx();
  y2 = posList->gety();
  w2 = *temp;
  *temp = 0x80000000;
  posList->reset();
  posList->setMax();
  temp = posList->getCurrentCount();
  x3 = posList->getx();
  y3 = posList->gety();
  w3 = *temp;

  //normalize
  w1 = (w1 - w3) + 1;
  w2 = (w2 - w3) + 1;
  w3 = 1;

  x = x1*w1 + x2*w2 + x3*w3;
  y = y1*w1 + y2*w2 + y3*w3;

  total = w1 + w2 + w3;

  x = x/total;
  y = y/total;

  point->x = x;
  point->y = y;
}

bool CRFMap::getPos4(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);
  if(posList == NULL) return false;
  if(!leastAPs(posList)) return false;
  addWeight(posList);
  if(!weightSignal(posList, sample)) return false;
  
  weightHistory(posList);
  posList->reset();
  posList->setMax();

	//int x, y;
	//int *temp;
	//int distance;
	//if(lastx > 0 && lasty > 0) {
	//	do {
	//		posList->reset();
	//		posList->setMax();
	//		temp = posList->getCurrentCount();
	//		*temp = 0x80000000;
	//		x = posList->getx();
	//		y = posList->gety();
	//		distance = pow(abs(x-lastx),2) + pow(abs(y-lasty),2);
	//	} while(distance > 5000);
	//}
	//else {
	//	posList->reset();
	//	posList->setMax();
	//	x = posList->getx();
	//	y = posList->gety();
	//}

  lastx = posList->getx();
  lasty = posList->gety();
  point->x = lastx;
  point->y = lasty;

  return true;
}

void CRFMap::weightHistory(positionLL *posList) {
  if(lastx >= 0 && lasty >= 0) {
    int x, y;
    int *temp;
    posList->reset();
    do {
      x = posList->getx();
      y = posList->gety();
      temp = posList->getCurrentCount();
      *temp = *temp - pow(abs(x-lastx),2)/300;
      *temp = *temp - pow(abs(y-lasty),2)/300;
    } while(posList->next());
  }
}
