
#include <string.h>
#include <iostream>
#include "CRFMap.h"
using namespace std;

#ifndef NULL
#define NULL 0x00000000
#endif

CRFMap::CRFMap(int sizex, int sizey) {
  fileSet = false;
  myTree = NULL;
  xsize = sizex;
  ysize = sizey;
  int i, j;
  macCount = new int **[sizex];
  for(i=0; i<xsize; i++) {
    macCount[i] = new int *[sizey];
    for(j=0; j<ysize; j++) {
      macCount[i][j] = new int;
      *macCount[i][j] = 0;
    }
  }
  mapFile = NULL;
}
CRFMap::~CRFMap() {
  int i, j;
  for(i=0; i<xsize; i++) {
    for(j=0; j<ysize; j++)
      delete macCount[i][j];
    delete[] macCount[i];
  }
  delete[] macCount;
  delete myTree;
}

bool CRFMap::SetCurrentFile(char * fileName) {
  if(fileName==NULL) return false;
  fileSet = true;
  if(mapFile == NULL) mapFile = new char[512];
  strcpy(mapFile, fileName);
  return true;
}

bool CRFMap::UndoLastAdd() { 
  return true;

}

bool CRFMap::ReadFromFile() {
  if(fileSet) {
    ifstream in;
    in.open(mapFile);
    if(!in) {
      return false;
    }
    if(myTree != NULL) {
      delete myTree;
    }
    char *mac = new char[18];
    int x, y, signal, noise, channel, count;
    in >> mac >> x >> y >> count >> signal >> noise >> channel;
    myTree = new macTree(x, y, mac, signal, noise, channel, count);
    *macCount[x][y] = *macCount[x][y] + 1;
    in >> x;
    while(x > 0) {
      in >> y >> count >> signal >> noise >> channel;
      myTree->addMac(x, y, mac, signal, noise, channel, count);
      *macCount[x][y] = *macCount[x][y] + 1;
      in >> x;
    }
    in >> mac;
    while(!in.eof()) {
      in >> x;
      while(x >= 0) {
	in >> y >> count >> signal >> noise >> channel;
	myTree->addMac(x, y, mac, signal, noise, channel, count);
	*macCount[x][y] = *macCount[x][y] + 1;
	in >> x;
      }
      in >> mac;
    }
  }
  return fileSet;
}

bool CRFMap::AddSample(int x, int y, CWirelessSample * sample) {
  if(sample==NULL || x<0 || y<0) return false;
  sample->Reset();
  CWirelessAPNode *current = sample->Current();
  if(current == NULL) return false;
  if(myTree == NULL) {
    myTree = new macTree(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
  }
  else {
    myTree->addMac(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
  }
  sample->Next();
  while(!sample->AtEnd()) {
    current = sample->Current();
    myTree->addMac(x, y, current->getMACAddr(), current->getSignal(), current->getNoise(), current->getChannel());
    *macCount[x][y] = *macCount[x][y] + 1;
    sample->Next();
  }
  sample->FreeMem();
  delete sample;
  return true;
}

bool CRFMap::GetPosition(CPoint * point, int mode, CWirelessSample * sample) {
  if(point==NULL || sample == NULL) return false;
  switch(mode) {
  case 0: return getPos0(point, sample);
  case 2: return getPos2(point, sample);
  default: return false;
  }
}

positionLL *CRFMap::getPosCounts(CPoint *point, CWirelessSample *sample) {
  //an array of counts for each position
  int ***count = new int **[xsize];
  int i, j;
  for(i=0; i<xsize; i++) {
    count[i] = new int *[ysize];
    for(j=0; j<ysize; j++) {
      count[i][j] = new int;
      *count[i][j] = 0;
    }
  }

  //Linked list of positions
  positionLL *posList = NULL;
  int x, y;

  sample->Reset();
  CWirelessAPNode *current;
  positionLL *currentPosLL;
  char *mac;

  while(!sample->AtEnd()) {
    current = sample->Current();
    mac = current->getMACAddr();
    currentPosLL = myTree->getMacPos(mac);
    if(currentPosLL != NULL) {
      currentPosLL->reset();
      currentPosLL->getPos(&x, &y);
      if(posList == NULL) {
	posList = new positionLL(x, y, 0, 0, 0);
	count[x][y] = posList->getCurrentCount();
      }
      else {
	if(*count[x][y] == 0) {
	  delete count[x][y];
	  count[x][y] = posList->addPos(x, y, 0, 0, 0);
	}
	else {
	  *count[x][y] = *count[x][y] + 1;
	}
      }
      while(currentPosLL->next()) {
	currentPosLL->getPos(&x, &y);
	if(*count[x][y] == 0) {
	  delete count[x][y];
	  count[x][y] = posList->addPos(x, y, 0, 0, 0);
	}
	else *count[x][y] = *count[x][y] + 1;
      }
    }
    sample->Next();
  }
  for(i=0; i<xsize; i++) {
    for(j=0; j<ysize; j++)
      if(*count[i][j] == 0) //don't want to delete the ones in poslist
	delete count[i][j];
    delete[] count[i];
  }
  delete[] count;
  return posList;
}

bool CRFMap::getPos2(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);
  if(posList == NULL) return false;
  posList->reset();
  posList->setMax();
  int maxCount = *posList->getCurrentCount();
  positionLL *newPosList = new positionLL(posList->getx(), posList->gety(), 0, 0, 0);
  int done = 0;
  if(!posList->next()) {
    done = 1;
  }
  while(!done) {
    posList->setMax();
    if(maxCount != *posList->getCurrentCount()) {
      done = 1;
    }
    else {
      newPosList->addPos(posList->getx(), posList->gety(), 0, 0, 0);
    }
    if(!posList->next()) {
      done = 1;
    }
  }
  int *temp;
  newPosList->reset();
  do {
    temp = newPosList->getCurrentCount();
    *temp = -*macCount[newPosList->getx()][newPosList->gety()];
  }while(newPosList->next());
  newPosList->reset();
  newPosList->setMax();
  newPosList->getPos((int *) &point->x, (int *) &point->y);

  delete newPosList;
  delete posList;
  return true;
}

bool CRFMap::getPos0(CPoint *point, CWirelessSample *sample) {
  positionLL *posList;
  posList = getPosCounts(point, sample);
  if(posList == NULL) return false;
  posList->reset();
  posList->setMax();
  posList->getPos((int *) &point->x,(int *) &point->y);

  delete posList;
  return true;
}

bool CRFMap::WriteToFile() {
  if(!fileSet) return false;
  ofstream out;
  out.open(mapFile);
  if(!out) {
    cout << "oops\n";
    return false;
  }
  myTree->writeToFile(&out);
  out.close();
  return true;
}




