#include <afxwin.h>
#include "CWirelessSample.h"
#include "mactree.h"

#ifndef __CRFMAP__
#define __CRFMAP__

class CRFMap {
public:
  CRFMap(int sizex, int sizey);
  ~CRFMap();

  // File access functions

  // SetCurrentFile points the RF map to a different file,
  // returns true on success
  bool SetCurrentFile(char * fileName);

  // ReadFromFile returns true on a successful read from file,
  // false otherwise
  bool ReadFromFile();

  // AddSample returns true on success, false otherwise
  bool AddSample(int x, int y, CWirelessSample * sample);

  // Getposition puts the position into the passed CPoint
  // returns true on success, false on failure
  // mode = 0 -> Simple inc if mac found algorithm
  // mode = 1 -> Interpolate position from data available
  //             IE do some weighted averaging or something
  //             to find position based on "nearby" known
  //             points.
  // mode = 2 -> inc if mac found dec if not algorithm
  // Write the position into the CPoint passed to the function
  bool GetPosition(CPoint * point, int mode, CWirelessSample * sample);

  // WriteToFile saves the CRFMap to the file
  bool WriteToFile();

  bool UndoLastAdd();

private:

  bool getPos0(CPoint *, CWirelessSample *);
  bool getPos2(CPoint *, CWirelessSample *);
  positionLL *getPosCounts(CPoint *, CWirelessSample *);

  // put whatever you want here
  bool fileSet; // don't need to keep this
  macTree *myTree;
  int ***macCount; //count of mac's for each x,y
  int xsize;
  int ysize;
  char *mapFile;
  fstream file;
};

#endif


