#include <afxcmn.h>
#include <process.h>

//#include "CWirelessSample.h"
#include "RFServerWindow.h"


MsgFromServer fromMe;
MsgToServer toMe;

void RunServer(void * me) {
	RFServer * rfserver = (RFServer *) me;
	RFServerWindow * Daddy = (RFServerWindow *) rfserver->daddy;
	CWirelessSample * sample;
	CWirelessAPNode * apnode;
	CPoint point;
	char buff[16];

	struct sockaddr_in fromAddr;

	int fromAddrSize = sizeof(fromAddr);
	
	int type;
	int length;
	int bytes;
	
	// do header setup
	fromMe.header.proto = htonl(0x90200902);
	fromMe.header.type = htonl(-1);
	fromMe.header.length = htonl(2);

	while(true) { // go until stopped
		
		// do the receive
		if((bytes = recvfrom(rfserver->sock,(char *) &toMe,sizeof(toMe),0,
			(struct sockaddr *) &fromAddr,&fromAddrSize))<0) {
			Daddy->PrintMessage("ERROR: Revcfrom() failed.\r");
			_endthread();
		}

		// okay, we've received a message, start checking it.
		if((bytes-12)%24!=0) {
			Daddy->PrintMessage("ERROR: Received an unexpected # of bytes: ");
			itoa(bytes,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage("\r");
			continue;
		}

		// verify proto
		if(ntohl(toMe.header.proto)!=0x90200902) {
			Daddy->PrintMessage("ERROR: Received an unexpected proto#: ");
			itoa(ntohl(toMe.header.proto),buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage("\r");
			continue;
		}

		// get type
		type = ntohl(toMe.header.type);
		if(type!=0 && type!=1) {
			Daddy->PrintMessage("ERROR: Received an unexpected type: ");
			itoa(type,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage("\r");
			continue;
		}

		// get length
		length = ntohl(toMe.header.length);
		if(length%6!=0) {
			Daddy->PrintMessage("ERROR: Received an unexpected length: ");
			itoa(length,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage("\r");
			continue;
		}

		// get the sample itself
		sample = new CWirelessSample(&toMe);
		sample->Reset();

		// if verbose, dump the sample
		if(Daddy->verbose) {
			Daddy->PrintMessage("Received sample:\r");
			while(!sample->AtEnd()) {
				apnode = sample->Current();
				Daddy->PrintMessage(apnode->getMACAddr());
				Daddy->PrintMessage("/C=");
				itoa(apnode->getChannel(),buff,10);
				Daddy->PrintMessage(buff);
				Daddy->PrintMessage("/S=");
				itoa(apnode->getSignal(),buff,10);
				Daddy->PrintMessage(buff);
				Daddy->PrintMessage("/N=");
				itoa(apnode->getNoise(),buff,10);
				Daddy->PrintMessage(buff);
				Daddy->PrintMessage("\r");
				sample->Next();
			}

			sample->Reset();
		}

		// find the position from the RF Map
		if(!rfserver->rfmap->GetPosition(&point,type,sample)) {
			Daddy->PrintMessage("ERROR: RFMap unable to resolve\r");
		}

		// send the position back to the sender
		fromMe.x = htonl(point.x);
		fromMe.y = htonl(point.y);

		// ready to be sent back
		
		Daddy->PrintMessage(" SENDING response to: ");
//		Daddy->PrintMessage((char *) ((LPCSTR)fromAddr));
		// DO WORK HERE TO GET NAME
		Daddy->PrintMessage(inet_ntoa(fromAddr.sin_addr));
		itoa(ntohs(fromAddr.sin_port),buff,10);
		Daddy->PrintMessage(":");
		Daddy->PrintMessage(buff);
		if(Daddy->verbose) {
			Daddy->PrintMessage("\r Position: (");
			itoa(point.x,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage(",");
			itoa(point.y,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage(")\r");
		} else {
			Daddy->PrintMessage("\r");
		}

		// delete the sample
		sample->FreeMem();
		delete sample;

		// send the reply here
		if((bytes = sendto(rfserver->sock,(char *) &fromMe,sizeof(fromMe),0,
			(struct sockaddr *) &fromAddr,sizeof(fromAddr))) != 20) {
			Daddy->PrintMessage("ERROR: Sent an unexpected number of bytes: ");
			itoa(bytes,buff,10);
			Daddy->PrintMessage(buff);
			Daddy->PrintMessage("\r");
		}


	}
}

RFServer::RFServer(CWnd *myDaddy,CRFMap *myMap) {
	daddy = myDaddy;
	rfmap = myMap;
	threadhandle = (void *)-1;
	sock = -1;
}

RFServer::~RFServer() {
	// if not shutdown, shut it down
	if(sock!=-1) closesocket(sock);
	WSACleanup();
}

bool RFServer::Init(UINT theport) {
	myPort = (unsigned short) theport;
	RFServerWindow * Daddy = (RFServerWindow *) daddy;
	// initialize WSA

	if(WSAStartup(MAKEWORD(2,0),&wsaData)!=0) {
		Daddy->PrintMessage("ERROR: WSAStartup() failed.\r");
		return false;
	}

	// okay, now create the socket itself

	if((sock=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP))<0) {
		Daddy->PrintMessage("ERROR: Unable to create socket.\r");
		return false;
	}

	// okay, now start setting up for the bind
	memset(&myAddr,0,sizeof(myAddr)); // zero initially
	myAddr.sin_family = AF_INET;
	myAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	myAddr.sin_port = htons(myPort);

	// bind!
	if(bind(sock,(struct sockaddr *) &myAddr,sizeof(myAddr))<0) {
		Daddy->PrintMessage("ERROR: Bind() failed.\r");
		closesocket(sock);
		sock = -1;
		return false;
	}

	
	Daddy->PrintMessage("Port: ");
	char buff[16];
	itoa(myPort,buff,10);
	Daddy->PrintMessage(buff);
	Daddy->PrintMessage("\r");
		
	return true;
}

bool RFServer::Run() {
	RFServerWindow * Daddy = (RFServerWindow *) daddy;
	
	threadhandle = (void *)_beginthread(RunServer,0,(void *) this);
	if(threadhandle==(void *)-1) return false;
	
	return true;

}

bool RFServer::Stop() {
	if(threadhandle!=(void *)-1) {
		if(TerminateThread(threadhandle,0)==TRUE) {
			threadhandle =(void *) -1;
			return true;
		} else {
			return false;
		}
	}
	else return false;
}

void RFServer::Shutdown() {
	if(threadhandle!=(void *)-1) {
		TerminateThread(threadhandle,0);
	}
	closesocket(sock);
	sock = -1;
	
}