
#include "RFServerWindow.h"
#include <process.h>

const char *commands[] = {"exit","load","run","stop","cls","verbose","help","gui","mode"};
const int nCommands = 9;

MsgFromServer fromMe;
MsgToServer toMe;

bool RFWindow::window_up;

void DieWithError(char * msg) {
	AfxMessageBox(msg);
	_endthread();
}

void RunServer(void * me) {
	char buff[12];
	RFServerWindow * rfwnd = (RFServerWindow *) me;

		if (WSAStartup(MAKEWORD(2, 0), &rfwnd->wsaData) != 0) /* Load Winsock 2.0 DLL */
		{
			AfxMessageBox("WSAStartup() failed");
			exit(1);
		}

		if ((rfwnd->sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
			DieWithError("socket() failed");

		/* Construct local address structure */
		memset(&rfwnd->echoServAddr, 0, sizeof(rfwnd->echoServAddr));   /* Zero out structure */
		rfwnd->echoServAddr.sin_family = AF_INET;                /* Internet address family */
		rfwnd->echoServAddr.sin_addr.s_addr = htonl(INADDR_ANY); /* Any incoming interface */
		rfwnd->echoServAddr.sin_port = htons(rfwnd->port);      /* Local port */
	
		/* Bind to the local address */
		if (bind(rfwnd->sock, (struct sockaddr *) &rfwnd->echoServAddr, sizeof(rfwnd->echoServAddr)) < 0)
		    DieWithError("bind() failed");
	
	    while (rfwnd->server_running)
	    {
	        /* Set the size of the in-out parameter */
	        rfwnd->cliLen = sizeof(rfwnd->echoClntAddr);
	
	        /* Block until receive message from a client */
	        if ((rfwnd->recvMsgSize = recvfrom(rfwnd->sock, (char*) &toMe, sizeof(toMe), 0,
	            (struct sockaddr *) &rfwnd->echoClntAddr, &rfwnd->cliLen)) < 0)
	            DieWithError("recvfrom() failed");

	        if(rfwnd->verbose)
			  rfwnd->PrintMessage("Handling a request--Sample is:\r");
	
			CPoint point;
			fromMe.header.length = htonl(2);
			fromMe.header.proto = htonl(0x90200902);
			fromMe.header.type = htonl(-1);
			
			CWirelessSample * sample = new CWirelessSample(&toMe);
			
			if(rfwnd->verbose) {
			sample->Reset();
			CWirelessAPNode * apnode;
			while(!sample->AtEnd()) {
				apnode = sample->Current();
				rfwnd->PrintMessage(" MAC: ");
				rfwnd->PrintMessage(apnode->getMACAddr());
				rfwnd->PrintMessage(" S=");
				itoa(apnode->getSignal(),buff,10);
				rfwnd->PrintMessage(buff);
				rfwnd->PrintMessage(" N=");
				itoa(apnode->getNoise(),buff,10);
				rfwnd->PrintMessage(buff);
				rfwnd->PrintMessage("\r");
				sample->Next();
			}
			}
			sample->Reset();
			
			if(rfwnd->verbose)
			  rfwnd->PrintMessage("***End of sample***\r");

			//rfwnd->rfmap->GetPosition(&point,ntohl(toMe.header.type),sample);
			rfwnd->rfmap->GetPosition(&point,rfwnd->getpos_mode,sample);

			sample->FreeMem();
			delete sample;

			fromMe.x = htonl(point.x);
			fromMe.y = htonl(point.y);

			if(rfwnd->verbose) {
			rfwnd->PrintMessage("Sending position: (");
			itoa(point.x,buff,10);
			rfwnd->PrintMessage(buff);
			itoa(point.y,buff,10);
			rfwnd->PrintMessage(",");
			rfwnd->PrintMessage(buff);
			rfwnd->PrintMessage(")\r");
			rfwnd->PrintMessage("SERVER: Client has been handled.\r>");
			}

			if(RFWindow::window_up) {
				rfwnd->rfwnd->moveLocation(point.x,point.y);
			}

	         /* Send received datagram back to the client */
	         if (sendto(rfwnd->sock, (char *)&fromMe, sizeof(fromMe), 0, (struct sockaddr *) &rfwnd->echoClntAddr, 
	                     sizeof(rfwnd->echoClntAddr)) != sizeof(fromMe))
	             DieWithError("sendto() sent a different number of bytes than expected");
	    }
	    
}

RFServerWindow::RFServerWindow() {
	if(Create(NULL,"CS/RADAR Server 1.0",WS_OVERLAPPEDWINDOW,
		CRect(550,0,1024,744),NULL)==FALSE) {

		AfxMessageBox("Failed to create window");
		exit(-1);
	}
	
	m_nLine = 0;			// empty text array
	m_anLen[m_nLine] = 0;	// empty line of text

	msg_printed = false;

	rfmap = new CRFMap(521,696);
	fileName = "";
	fileOpen = false;
	server_running = false;
	verbose = true;

	rfwnd = NULL;

	getpos_mode = 2;

	RFWindow::window_up = false;

	PrintMessage(">");
}

RFServerWindow::~RFServerWindow() {
	
}

afx_msg void RFServerWindow::OnDestroy() {
	delete rfmap;
}

afx_msg void RFServerWindow::OnPaint() {
	CPaintDC dc(this);

	int nPosition = m_anLen[m_nLine]++;

	m_asText[m_nLine][nPosition] = '_'; // a cursor

	for(int ln=0; ln<=m_nLine; ln++) // paint lines
		dc.TextOut(1,LINE_HEIGHT*ln,
					m_asText[ln],m_anLen[ln]);

	m_anLen[m_nLine]--; // remove the cursor

}  

afx_msg void RFServerWindow::OnChar(UINT uChar, UINT uRepCnt, UINT uFlg) {
	
	switch(uChar) {
	case '\r':
		m_nLine++;
		if(m_nLine>=LINES)
			m_nLine=0;
		m_anLen[m_nLine]=0;
		
		if(!msg_printed) {
			// remove the cursor before calling parse
			int parse_index = (m_nLine==0) ? LINES-1 : m_nLine -1;
			m_asText[parse_index][m_anLen[parse_index]]=0;
			parse(m_asText[(m_nLine==0) ? LINES-1 : m_nLine -1]);

		}
		break;
	case '\b':
		if(m_anLen[m_nLine]>0)
			m_anLen[m_nLine]--;

		break;
	default:
		int nPosition = m_anLen[m_nLine]++;
		m_asText[m_nLine][nPosition] = uChar;
		if(m_anLen[m_nLine]>=LINE_LENGTH) {
			if(++m_nLine>=LINES) {
				m_nLine = 0;
			}

			m_anLen[m_nLine]=0;
		}
	
	}

	InvalidateRect(NULL);
}

void RFServerWindow::PrintMessage(const char * message) {
	msg_printed = true;
	int i=0;
	while(message[i]!=NULL) {
		OnChar(message[i],0,0);
		i++;
	}
	msg_printed = false;
}

// all console commands handled here
void RFServerWindow::parse(char * theline) {
	int i;
	char * line = new char[LINE_LENGTH];
	strcpy(line,theline);
	char * token = strtok(&line[0],"> \t\n");
	
	switch(doTokenComparisons(token)) {
	case 0: // exit
		PrintMessage("CONSOLE: Goodbye.\r");
		// do whatever else you need to do on shutdown here

		SendMessage(WM_CLOSE);
		break;
	case 1: // load
		if(server_running) {
			PrintMessage("CONSOLE: Cannot load: Server running.\r");
			break;
		}

		token = strtok(NULL," \t\n");
		if(token==NULL) { // first token should be filename
			PrintMessage("CONSOLE: Syntax is: load <filename>\r");
			break;
		}
		if(!rfmap->SetCurrentFile(token)) {
			PrintMessage("CONSOLE: Unable to select file: ");
			PrintMessage(token);
			PrintMessage("\r");
			break;
		}
		if(!rfmap->ReadFromFile()) {
			PrintMessage("CONSOLE: Unable to read from file: ");
			PrintMessage(token);
			PrintMessage("\r");
			break;
		}

		// add code here to save the filename for the gui

		IgnoreOtherTokens();

		fileName = token;
		fileOpen = true;

		PrintMessage("CONSOLE: Successfully loaded RF Map from file: ");
		PrintMessage((char *) ((LPCSTR)fileName));
		PrintMessage("\r");

		break;
	case 2:  // run!
		token = strtok(NULL," \t\n");
		if(token==NULL) { // first token should be port number
			PrintMessage("CONSOLE: Syntax is: run <port#>\r");
			break;
		}

		port = atoi(token);
		if(port<=0 || port > 65535) {
			PrintMessage("CONSOLE: Invalid port:");
			PrintMessage(token);
			PrintMessage("\r");
			break;
		}

		if(fileOpen) {
			PrintMessage("CONSOLE: Starting server: Port ");
			PrintMessage(token);
			PrintMessage("\r");

		} else {
		    PrintMessage("CONSOLE: Cannot start server: No RF Map loaded.\r");
			break;
		}

		IgnoreOtherTokens();

		// do whatever other magic here
		server_running = true;
		_beginthread(RunServer,0,(void *) this);
		

		break;

	case 3: // stop
		if(server_running) {
			PrintMessage("CONSOLE: Stopping server.\r");
			// do whatever magic here to stop server
			server_running = false;

		} else {
			PrintMessage("CONSOLE: Server not running.\r");
		}
		IgnoreOtherTokens();
		break;
	case 4: // cls
		IgnoreOtherTokens();
		// set all line lengths to 0
		for(i=0;i<LINES;i++)
			m_anLen[i] = 0;
		// set cursor position to 0
		m_nLine = 0;
		break;

	case 5: // verbose
		token = strtok(NULL," \t\n");
		if(stricmp(token,"on")==0) {
			verbose = true;
			PrintMessage("CONSOLE: Verbose mode on.\r");
		} else if(stricmp(token,"off")==0) {
			verbose = false;
			PrintMessage("CONSOLE: Verbose mode off.\r");
		} else {
			PrintMessage("CONSOLE: Syntax is: verbose [on|off]\r");
		}

		IgnoreOtherTokens();

		break;

	case 6: // help
		IgnoreOtherTokens();
		OnChar('>',0,0);
		OnChar('c',0,0);
		OnChar('l',0,0);
		OnChar('s',0,0);
		OnChar('\r',0,0); // force a CLS
		PrintMessage("CONSOLE: CS/RADAR Server command summary:\r");
		PrintMessage("  exit             - shutdown application\r");
		PrintMessage("  load <filename>  - load RF data from file\r");
		PrintMessage("  run <port#>      - start server\r");
		PrintMessage("  stop             - stop server\r");
		PrintMessage("  cls				 - clear screen\r");
		PrintMessage("  verbose [on|off] - Echo client requests\r");
		PrintMessage("  help             - Print this message\r");
		PrintMessage("  gui [on|off]     - Turn map on/off\r");
		break;

	case 7: // gui
		token = strtok(NULL," \t\n");
		IgnoreOtherTokens();

		if(stricmp(token,"on")==0) {
			if(RFWindow::window_up) {
				PrintMessage("CONSOLE: GUI already running.\r");
				break;
			}
	
			rfwnd = new RFWindow;
			RFWindow::window_up = true;
			rfwnd->ShowWindow( SW_SHOW );
			rfwnd->UpdateWindow();

			PrintMessage("CONSOLE: GUI Enabled.\r");
		} else if(stricmp(token,"off")==0) {
			if(RFWindow::window_up) {
				RFWindow::window_up = false;
				rfwnd->SendMessage(WM_CLOSE);
				PrintMessage("CONSOLE: GUI closed.\r");
			} else {
				PrintMessage("CONSOLE: GUI not running.\r");
			}
		} else {
			PrintMessage("CONSOLE: Unrecognized GUI Command: ");
			PrintMessage(token);
			PrintMessage("\r");
		}
		break;
	case 8:  // mode
		token = strtok(NULL," \t\n");
		if(token==NULL||token[0]>'9'||token[0]<'0') {
			PrintMessage("CONSOLE: Mode requires an integer parameter\r");
			break;
		}
		IgnoreOtherTokens();

		getpos_mode = atoi(token);

		PrintMessage("CONOLE: Lookup mode changed to ");
		PrintMessage(token);
		PrintMessage("\r");

		break;
	default: // no matches
		PrintMessage("CONSOLE: Unable to parse token: ");
		PrintMessage(token);
		PrintMessage("\r");
		break;
	}

	delete line;
	PrintMessage(">");

}

void RFServerWindow::IgnoreOtherTokens() {
	char * token;
	token = strtok(NULL," \t\n");
	while(token!=NULL) {
		PrintMessage("CONSOLE: Ignoring extra token: ");
		PrintMessage(token);
		PrintMessage("\r");
		token = strtok(NULL," \t\n");
	}	
}

int RFServerWindow::doTokenComparisons(char * token) {
	int i;
	for(i=0;i<nCommands;i++) {
		if(stricmp(commands[i],token)==0) break;
	}

	return i;
}

BEGIN_MESSAGE_MAP( RFServerWindow, CFrameWnd )
  ON_WM_CHAR()
  ON_WM_PAINT()
  ON_WM_DESTROY()
END_MESSAGE_MAP()

