#include <stdio.h>

long compares;

int suffix_search(char * pattern, int pattern_length, int suffix_length) {
  char * suffix = pattern + pattern_length - suffix_length;
  int suffix_pos = 1-suffix_length;
  bool match = false;
  int last_match = pattern_length - suffix_length;
  for(suffix_pos;suffix_pos<pattern_length-suffix_length;suffix_pos++) {
    for(int i=0;i<suffix_length;i++) {

      if(suffix_pos+i<0) {
        match = true;

      } else if (suffix[i]==pattern[suffix_pos+i]) {
        compares++;

        // characters match or suffix off front of pattern
        match = true;

      } else {
        match = false;
        compares++;
        break;

      }

    }

    if(match)
      if(!(suffix_pos<1||pattern[suffix_pos-1]==suffix[-1])) match = false;

    if(match) last_match = pattern_length - suffix_length - suffix_pos;

  }

  return last_match;
}

long boyerMoore(char * text, int text_length, char * pattern, int pattern_length) {
  pattern_length--; // account for null termination
  compares = 0;
  int * badchar_table = new int[256]; // use ASCII for indexing

  // initialize badchar_table to shift of full pattern
  for(int i=0;i<256;i++) badchar_table[i]=pattern_length;

  // do preprocessing
  for(int i=0; i<pattern_length-2;i++) {
    badchar_table[pattern[i]] = pattern_length - i - 1;
  }

  int * goodsuffix_table = new int[pattern_length];
  goodsuffix_table[0] = 1;
  int depth;

  for(int i=1;i<pattern_length;i++) {
    depth = suffix_search(pattern, pattern_length, i);

    goodsuffix_table[i] = depth;
  }

  int right_end = pattern_length - 1;
  int num_matched_chars;
  char c;
  int max_shift;

  while(right_end < text_length) {

    num_matched_chars = 0;

    c = text[right_end-num_matched_chars];

    while(pattern[pattern_length-1-num_matched_chars]==c &&
          num_matched_chars < pattern_length ) {
      compares++;
      num_matched_chars++;
      c = text[right_end-num_matched_chars];

    }

    compares++; // once for failed comparison

    if(num_matched_chars==pattern_length) {
      // a match has been found
      printf("MATCH FOUND AT BYTE %i\n",right_end-pattern_length+2);
      return compares;
    } else {
      // not a match, move on
      if(badchar_table[c]-num_matched_chars<1) {
        max_shift = 1;
      } else {
        max_shift = badchar_table[c]-num_matched_chars;
      }

      if(max_shift<goodsuffix_table[num_matched_chars])
        max_shift = goodsuffix_table[num_matched_chars];
      right_end += max_shift;
    }

  }

  printf("NO MATCH\n");
  return compares;

}

long stringCompare(char * text, int text_length, char * pattern, int pattern_length) {
  return boyerMoore(text,text_length,pattern,pattern_length);
}


