#include <afxdlgs.h>
#include <math.h>
#include "RFWindow.h"

RFWindow::RFWindow() {
	if(Create(NULL,TITLE_STRING,WS_OVERLAPPEDWINDOW,
		CRect(0,0,521,724),NULL,NULL)==FALSE) {

		AfxMessageBox("Failed to create window");
		exit(-1);
	}
	
	if(bmp_map.LoadBitmap("map")==0) {
		AfxMessageBox("Failed to load map bitmap");
		exit(-1);
	}
	CClientDC dc(this);
	if(memDC.CreateCompatibleDC(&dc)==0) {
		AfxMessageBox("Failed to create compatible device context");
		exit(-1);
	}
	memDC.SelectObject(&bmp_map);
	
	if(bmp_x.LoadBitmap("redx")==0) {
		AfxMessageBox("Failed to load map bitmap");
		exit(-1);
	}
	if(xDC.CreateCompatibleDC(&dc)==0) {
		AfxMessageBox("Failed to create compatible device context");
		exit(-1);
	}
	xDC.SelectObject(&bmp_x);

	infile = NULL;
	currentFileName = "";

	truex[0] = truey[0] = predx[0] = predy[0] = -1;

	next_sample = 0;

	draw_truepath = draw_numbers = draw_background = draw_predpath =
		draw_elipses = draw_samples = true;

}

RFWindow::~RFWindow() {
	
}

afx_msg void RFWindow::OnPaint() {
	CPaintDC dc(this);
	CPen r_pen;
	r_pen.CreatePen(PS_SOLID,1,RGB(255,0,0));
	dc.SelectObject(&r_pen);
	CPen g_pen;
	g_pen.CreatePen(PS_SOLID,1,RGB(255,0,255));
	CPen dg_pen;
	dg_pen.CreatePen(PS_SOLID,3,RGB(0,128,0));
	CPen b_pen;
	b_pen.CreatePen(PS_SOLID,1,RGB(0,0,255));

	// draw the map itself
	if(draw_background) dc.BitBlt( 0,0,521,696,&memDC,0,0,SRCCOPY);
	else {
		dc.FillSolidRect(0,0,521,696,RGB(255,255,255));
		/*
		for(int i=0;i<522;i++) {
			for(int j=0;j<697;j++) {
				dc.SetPixel(i,j,RGB(255,255,255));
			}
		}
		*/
	}

	// draw the samples
	if(draw_samples) {
		dc.SelectObject(&dg_pen);
		for(int i=0;i<next_sample;i++) {
			dc.MoveTo(sampx[i]-1,sampy[i]-1);
			dc.LineTo(sampx[i],sampy[i]);

		}
	}

	char buff[8];

	// draw the predictions and actuals
	int current = 0;

	if(draw_truepath) {
		dc.SelectObject(&g_pen);
		while(truex[current]!=-1) {
			// draw the path
			dc.MoveTo(truex[current],truey[current]);
			if(truex[current+1]!=-1)
				dc.LineTo(truex[current+1],truey[current+1]);
			
			current++;
		}
		current = 0;
	}

	if(draw_predpath) {
		dc.SelectObject(&b_pen);
		while(predx[current]!=-1) {
			// draw the predicted path
			dc.MoveTo(predx[current],predy[current]);
			if(predx[current+1]!=-1)
				dc.LineTo(predx[current+1],predy[current+1]);
			current++;
		}
		current = 0;
	}

	if(draw_numbers){
		while(truex[current]!=-1) {
		// label the path

			itoa(current+1,buff,10);
			dc.TextOut(truex[current]-7,truey[current]-10,buff);
		
			current++;
		}

		current = 0;

	}

	if(draw_elipses) {
	while(truex[current]!=-1) {

		// draw the X at the predicted location
		dc.BitBlt( predx[current]-3,predy[current]-2,
					5,6,&xDC,0,0,SRCCOPY);

		// draw a line from predicted location to 
		// actual location
		dc.SelectObject(&r_pen);
		dc.MoveTo(predx[current],predy[current]);
		//dc.LineTo(truex[current],truey[current]);
		
		dc.SetArcDirection( (dc.GetArcDirection()==AD_COUNTERCLOCKWISE)?
			AD_CLOCKWISE : AD_COUNTERCLOCKWISE);


		// if pred is left of true
		if(predx[current]<=truex[current]) {
			// pred is left of true
			if(predy[current]<=truey[current]) {
				// Pred is UL, true is LR
				dc.ArcTo(predx[current]-5,
						 predy[current]-5,
						 truex[current]+5,
						 truey[current]+5,
						 predx[current],
						 predy[current],
						 truex[current],
						 truey[current]);
						 
			} else {
				// pred is LL, true is UR
				dc.ArcTo(predx[current]-5,
						 truey[current]-5,
						 truex[current]+5,
						 predy[current]+5,
						 predx[current],
						 predy[current],
						 truex[current],
						 truey[current]);
						 

			}
		} else {
			// true is left of pred
			if(predy[current]<=truey[current]) {
				// pred is UR, true is LL
				dc.ArcTo(truex[current]-5,
						 predy[current]-5,
						 predx[current]+5,
						 truey[current]+5,
						 predx[current],
						 predy[current],
						 truex[current],
						 truey[current]);
						 

			} else {
				// pred is LR, true is UL
				dc.ArcTo(truex[current]-5,
						 truey[current]-5,
						 predx[current]+5,
						 predy[current]+5,
						 predx[current],
						 predy[current],
						 truex[current],
						 truey[current]); 
				
			}
		}

		current++;
	}
	}

/*
	if(draw_location) {
		dc.BitBlt( myLocation.x-3,myLocation.y-2,5,6,&xDC,0,0,SRCCOPY);

		dc.MoveTo(oldLocation.x,oldLocation.y);
		dc.LineTo(myLocation.x,myLocation.y);

		if( myLocation.x > 149 && myLocation.x < 194 &&
			myLocation.y > 250 && myLocation.y < 343) { // near megerian's office
			dc.BitBlt(10,10,150,200,&megerianDC,0,0,SRCCOPY);
			dc.TextOut(10,210,"\"Our fearless leader\"");
		}
	}
*/
/*
	// code for printing distance
	char buff[12];
	int delta_x = xpos - myLocation.x;
	int delta_y = ypos - myLocation.y;
	double distance = sqrt((double) delta_x*delta_x + delta_y*delta_y);
	gcvt(distance,9,buff);
	dc.TextOut(0,0,buff);

	// added code for point coloring
	int current = 0;
	CPen g_pen;
	g_pen.CreatePen(PS_SOLID,2,RGB(0,128,0));
	dc.SelectObject(&g_pen);
	while(x[current]!=-1) {
		dc.MoveTo(x[current]-1,y[current]-1);
		dc.LineTo(x[current]+1,y[current]+1);
		dc.MoveTo(x[current]+1,y[current]-1);
		dc.LineTo(x[current]-1,y[current]+1);
		current++;
	}
*/

}  

afx_msg void RFWindow::OnDestroy() {
	if(infile!=NULL) {
		fclose(infile);
		infile = NULL;
	}
	exit(0);
}

void RFWindow::Save() {
	CFileDialog fd(FALSE,		// TRUE for open, FALSE for save as
				   NULL,		// Default extension
				   NULL,			// Initial file name
				   OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,				// Flag values
				   NULL,
				   NULL);
	if(fd.DoModal()==IDCANCEL) return;

	currentFileName = fd.GetFileName();

	FILE * outfile = fopen(currentFileName,"w");
	if(outfile==NULL) {
		AfxMessageBox("Unable to write to file.");
		return;
	}

	char buff[64];

	for(int i=0;i<next_sample;i++) {
		sprintf(buff,"%i,%i\n",sampx[i],sampy[i]);
		fputs(buff,outfile);

	}

	fflush(outfile);
	fclose(outfile);

}

void RFWindow::LoadSamples(char * filename) {
	FILE * in = fopen(filename,"r");
	if(in==NULL) {
		AfxMessageBox("Unable to read from samples file");
		return;
	}

	char buff_in[64];
	char * token;
	int current = 0;

	while(fgets(buff_in,64,in)) {
		token = strtok(buff_in," ,\t\n");
		sampx[current] = atoi(token);
		token = strtok(NULL," ,\t\n");
		sampy[current] = atoi(token);
		current++;
		if(current==1024) {
			AfxMessageBox("Number of samples in file exceeds 1024: truncating");
			break;
		}
	}

	sprintf(buff_in,"Read %i samples from file.",current);

	next_sample = current;

	fclose(in);

	AfxMessageBox(buff_in);
}

void RFWindow::Load() {
	if(infile!=NULL) {
		fclose(infile);
		infile = NULL;
	}
	
	CFileDialog fd(TRUE,			// TRUE for open, FALSE for save as
				   NULL,		// Default extension
				   NULL,			// Initial file name
				   OFN_HIDEREADONLY | OFN_FILEMUSTEXIST,				// Flag values
				   NULL,
				   NULL);
	if(fd.DoModal()==IDCANCEL) return;

	currentFileName = fd.GetFileName();

	// hack for allowing sample files too
	if(currentFileName.Find(".samples")!=-1) {
		LoadSamples(currentFileName.GetBuffer(64));
		currentFileName.ReleaseBuffer();
		return;
	}

	// do loading here
	int current = 0;
	int ax, ay, px, py;
	
	infile = fopen(currentFileName,"r");
	if(infile==NULL) {
		AfxMessageBox("Unable to open selected file.");
		return;
	}

	char buff_in[1024];
	char * token;

	// True x, True y, Est x, Est y, distance

	while(fgets(buff_in,1024,infile)) {
		token = strtok(buff_in," ,\t\n");
		if(token==NULL) continue;
		ax = atoi(token);

		token = strtok(NULL," ,\t\n");
		if(token==NULL) continue;
		ay = atoi(token);

		token = strtok(NULL," ,\t\n");
		if(token==NULL) continue;
		px = atoi(token);

		token = strtok(NULL," ,\t\n");
		if(token==NULL) continue;
		py = atoi(token);

		truex[current] = ax;
		truey[current] = ay;
		predx[current] = px;
		predy[current] = py;
		current++;

		if(current>=1023) break;

	}

	truex[current] = -1;
	truey[current] = -1;
	predx[current] = -1;
	predy[current] = -1;

	itoa(current,buff_in,10);
	currentFileName = "Read in ";
	currentFileName+=buff_in;
	currentFileName+=" useful lines.";
	AfxMessageBox(currentFileName);

	fclose(infile);
	infile = NULL;

	InvalidateRect(NULL);

}

afx_msg void RFWindow::OnLButtonDown(UINT nFlags, CPoint point) {
	if(!(nFlags & MK_RBUTTON)) {
		// if R Button not held down
		char buff[8];
		CString msg = "(";
		itoa(point.x,buff,10);
		msg+=buff;
		msg+=",";
		itoa(point.y,buff,10);
		msg+=buff;
		msg+=")";
		AfxMessageBox(msg);
		return;
	}

	// add a point to the sample
	if(AddSample(point))
		InvalidateRect(CRect(point.x-3,point.y-3,point.x+3,point.y+3));
	else {
		AfxMessageBox("Out of room for sample!");
	}
}

afx_msg void RFWindow::OnMButtonDown(UINT nFlags, CPoint point) {
	if(AfxMessageBox("Undo last add?",MB_YESNO)==IDNO) return;
	if(!UndoAdd()) {
		AfxMessageBox("Cannot undo add!");
	}

		InvalidateRect(NULL);

}

bool RFWindow::UndoAdd() {
	if(next_sample==0) return false;
	next_sample--;

	return true;
}

bool RFWindow::AddSample(CPoint point) {
	if(next_sample==1024) return false;
	sampx[next_sample] = point.x;
	sampy[next_sample] = point.y;
	next_sample++;
	return true;
}

// hotkeys:
// l	Load
// n	Toggle draw numbers
// m	Toggle draw map
// t	Toggle draw true path
// e	Toggle draw elipses
// p	Toggle draw predicted path
// s	Toggle draw samples
// S	Save the samples

afx_msg void RFWindow::OnChar(UINT uChar, UINT nRepCnt,UINT nFlags) {
	switch(uChar) {
	case 'l':
		Load();
		break;
	case 'n':
		draw_numbers = !draw_numbers;
		break;
	case 'm':
		draw_background = !draw_background;
		break;
	case 't':
		draw_truepath = !draw_truepath;
		break;
	case 'e':
		draw_elipses = !draw_elipses;
		break;
	case 'p':
		draw_predpath = !draw_predpath;
		break;
	case 's':
		draw_samples = !draw_samples;
		break;
	case 'S':
		Save();
		break;
	default:
		break;
	}

	InvalidateRect(NULL);
}

BEGIN_MESSAGE_MAP( RFWindow, CFrameWnd )

  ON_WM_PAINT()
  ON_WM_DESTROY()

  ON_WM_CHAR()
  ON_WM_LBUTTONDOWN()
  ON_WM_MBUTTONDOWN()

END_MESSAGE_MAP()

